"""Asap module ParallelListOfAtoms.

Defines the parallel list of atoms object (`ParallelAtoms`), and a factory
method for creating them (`MakeParallelAtoms`).

Importing this module also installs a Python exit function causing
MPI_Abort to be called if an uncaught exception occurs.
"""


__docformat__ = "restructuredtext en"

import ase
import ase.units
from asap3 import _asap
from asap3.Internal.BuiltinPotentials import AsapPotential
import asap3.mpi
import numpy as np
import pickle
import sys, time
import ase.parallel
import numbers

class ParallelPotential(_asap.ParallelPotential, AsapPotential):
    def get_stress(self, atoms):
        stress = self.get_virial(atoms)
        p = atoms.get_momenta()
        masses = atoms.get_masses()
        invmass = 1.0 / masses
        dynstress = np.zeros_like(stress)
        for alpha in range(3):
            for beta in range(alpha, 3):
                dynstress[self._stresscomp[alpha,beta]] = -(p[:,alpha] * p[:,beta] * invmass).sum()
        asap3.mpi.world.sum(dynstress)
        stress = (stress + dynstress) / atoms.get_volume()
        return stress

    
class ParallelAtoms(ase.Atoms):
    """Atoms class for parallel Asap simulations.

    It is recommended to create ParallelAtoms objects using
    `MakeParallelAtoms`.
    """
    parallel = 1
    def __init__(self, nCells, comm, atoms, cell=None, pbc=None,
                 distribute=True):
        """Create a ParallelAtoms object.

        WARNING: ParallelAtoms object should normally not be created
        explicitly.  Use MakeParallelAtoms instead.
        """
        self.info = {}
        self.nCells = np.array(nCells, int)
        self.comm = comm
        if self.nCells.shape != (3,):
            raise ValueError("ParallelAtoms: nCells must be 3 integers.")
        self._calc = None
        # Extract all data from the atoms object.  Not nice :-)
        self.arrays = {}
        for name in atoms.arrays.keys():
            self.arrays[name] = atoms.arrays[name].copy()
        assert self.arrays["positions"].dtype == np.dtype(float)
        assert self.arrays["positions"].shape == (len(atoms), 3)
        assert self.arrays["numbers"].shape ==  (len(atoms),)

        if cell is not None:
            self.cell = cell
        else:
            self.cell = atoms.get_cell()
        if pbc is not None:
            self.pbc = pbc
        else:
            self.pbc = atoms.get_pbc()
        self.info = {}

        self.ghosts = {}
        self.ghosts["positions"] = np.zeros((0,3), float)
        self.ghosts["numbers"] = np.zeros(0, self.arrays["numbers"].dtype)

        # Now make the IDs
        mynatoms = np.array([len(self)])
        natoms_all = np.zeros(self.comm.size, int)
        self.comm.all_gather(mynatoms, natoms_all)
        if "ID" not in self.arrays:
            firstID = sum(natoms_all[:self.comm.rank])
            self.arrays["ID"] = np.arange(firstID, firstID+len(atoms))
        self.total_number_of_atoms = sum(natoms_all)

        # Atoms should have no constraints
        self.set_constraint(None)
        
        if distribute:
            self.distribute()

    def distribute(self):
        _asap.DistributeAtoms(self)

    def get_number_of_atoms(self):
        n = len(self)
        return self.comm.sum(n)
        
    def get_list_of_elements(self):
        """Get a list of elements.

        The list is cached to prevent unnecessary communication.
        """
        try:
            return self.listofelements
        except AttributeError:
            z = self.get_atomic_numbers()
            present = np.zeros(100, int)
            zmax = z.max()
            zmin = z.min()
            present[zmin] = present[zmax] = 1
            for i in range(zmin+1, zmax):
                if np.equal(z, i).any():
                    present[i] = 1
            self.comm.sum(present)
            self.listofelements = []
            for i, p in enumerate(present):
                if p:
                    self.listofelements.append(i)
            return self.listofelements

    def set_atomic_numbers(self, numbers):
        """Set the atomic numbers."""
        try:
            # Discard the cached list of elements
            del self.listofelements
        except AttributeError:
            pass
        ase.Atoms.set_atomic_numbers(self, numbers)

    def get_ids(self):
        """Get the atom IDs in a parallel simulation."""
        return self.arrays["ID"].copy()

    def is_master(self):
        """Return 1 on the master node, 0 on all other nodes."""
        return (self.comm.rank == 0)

    def get_comm(self):
        return self.comm
    
    def wrap_calculator(self, calc):
        "Make an ASAP calculator compatible with parallel simulations."
        try:
            parallelOK = calc.supports_parallel()
        except AttributeError:
            parallelOK = False
        if not parallelOK:
            raise ValueError("The calculator does not support parallel ASAP calculations.")
        return ParallelPotential(calc)

    def set_calculator(self, calc, wrap=True):
        """Sets the calculator in a way compatible with parallel simulations.
        
        calc: 
            The Calculator to be used.  Normally only Asap calculators will work.
            
        wrap (optional, default=True):
            Indicates if a calculator should be wrapped in a ParallelCalculator object.  
            Wrapping is the default, and should almost always be used, the only exception
            being if the Calculator is implemented as a Python object wrapping an Asap
            calculator, in which case the Asap calculator should first be wrapped in
            a ParallelCalculator object (use atoms.wrap_calculator) and this one should then
            be used by the Python calculator.  The Python calculator is then attached
            without being wrapped again.
        """
        if wrap:
            parcalc = self.wrap_calculator(calc)
        else:
            parcalc = calc
        ase.Atoms.set_calculator(self, parcalc)

    def get_kinetic_energy(self):
        local_ekin = ase.Atoms.get_kinetic_energy(self)
        return self.comm.sum(local_ekin)
    
    def get_temperature(self):
        """Get the temperature. in Kelvin"""
        ekin = self.get_kinetic_energy() / self.get_number_of_atoms()
        return ekin / (1.5 * ase.units.kB)

    def get_ghost_positions(self):
        return self.ghosts['positions'].copy()
    
    def get_ghost_atomic_numbers(self):
        return self.ghosts['numbers'].copy()
    
    def get_center_of_mass(self, scaled=False):
        """Get the center of mass.

        If scaled=True the center of mass in scaled coordinates
        is returned."""
        m = self.get_masses()
        rsum = np.dot(m, self.arrays['positions'])
        msum = m.sum()
        data = np.zeros(4)
        data[:3] = rsum
        data[3] = msum
        self.comm.sum(data)
        com = data[:3] / data[3]
        if scaled:
            return np.linalg.solve(self._cell.T, com)
        else:
            return com

    # We need to redefine __getitem__ since __init__ does not
    # take the expected arguments.
    def __getitem__(self, i):
        """Return a subset of the atoms.

        i -- scalar integer, list of integers, or slice object
        describing which atoms to return.

        If i is a scalar, return an Atom object. If i is a list or a
        slice, return an Atoms object with the same cell, pbc, and
        other associated info as the original Atoms object. The
        indices of the constraints will be shuffled so that they match
        the indexing in the subset returned.

        The returned object is an ordinary Atoms object, not a
        ParallelAtoms object.
        """
        if isinstance(i, numbers.Integral):
            natoms = len(self)
            if i < -natoms or i >= natoms:
                raise IndexError('Index out of range.')

            return ase.Atom(atoms=self, index=i)

        import copy
        from ase.constraints import FixConstraint, FixBondLengths

        atoms = ase.Atoms(cell=self._cell, pbc=self._pbc, info=self.info)

        atoms.arrays = {}
        for name, a in self.arrays.items():
            atoms.arrays[name] = a[i].copy()

        # Constraints need to be deepcopied, since we need to shuffle
        # the indices
        atoms.constraints = copy.deepcopy(self.constraints)
        condel = []
        for con in atoms.constraints:
            if isinstance(con, (FixConstraint, FixBondLengths)):
                try:
                    con.index_shuffle(self, i)
                except IndexError:
                    condel.append(con)
        for con in condel:
            atoms.constraints.remove(con)
        return atoms

def MakeParallelAtoms(atoms, nCells, cell=None, pbc=None,
                      distribute=True):
    """Build parallel simulation from serial lists of atoms.

    Call simultaneously on all processors.  Each processor having
    atoms should pass a list of atoms as the first argument, or None
    if this processor does not contribute with any atoms.  If the
    cell and/or pbc arguments are given, they must be given on
    all processors, and be identical.  If it is not given, a supercell
    is attempted to be extracted from the atoms on the processor with
    lowest rank.

    This is the preferred method for creating parallel simulations.
    """
    mpi = asap3.mpi
    #comm = mpi.world.duplicate()
    comm = mpi.world

    # Sanity check: is the node layout reasonable
    nNodes = nCells[0] * nCells[1] * nCells[2]
    if nNodes != comm.size:
        raise RuntimeError("Wrong number of CPUs: %d != %d*%d*%d" %
                           (comm.size, nCells[0], nCells[1], nCells[2]))
    t1 = np.zeros((3,))
    t2 = np.zeros((3,))
    comm.min(t1)
    comm.max(t2)
    if (t1[0] != t2[0] or t1[1] != t2[1] or t1[2] != t2[2]):
        raise RuntimeError("CPU layout inconsistent.")

    # If pbc and/or cell are given, they may be shorthands in need of
    # expansion.
    if pbc:
        try:
            plen = len(pbc)
        except TypeError:
            # It is a scalar, interpret as a boolean.
            if pbc:
                pbc = (1,1,1)
            else:
                pbc = (0,0,0)
        else:
            if plen != 3:
                raise ValueError("pbc must be a scalar or a 3-sequence.")
    if cell:
        cell = array(cell)  # Make sure it is a numeric array.
        if cell.shape == (3,):
            cell = array([[cell[0], 0, 0],
                          [0, cell[1], 0],
                          [0, 0, cell[2]]])
        elif cell.shape != (3,3):
            raise ValueError("Unit cell must be a 3x3 matrix or a 3-vector.")

    # Find the lowest CPU with atoms, and let that one distribute
    # which data it has.  All other CPUs check for consistency.
    if atoms is None:
        hasdata = None
        mynum = comm.size
    else:
        hasdata = {}
        for name in atoms.arrays.keys():
            datatype = np.sctype2char(atoms.arrays[name])
            shape = atoms.arrays[name].shape[1:]
            hasdata[name] = (datatype, shape)
        mynum = comm.rank
        if pbc is None:
            pbc = atoms.get_pbc()
        if cell is None:
            cell = atoms.get_cell()
    root = comm.min(mynum)   # The first CPU with atoms
    # Now send hasdata, cell and pbc to all other CPUs
    package = pickle.dumps((hasdata, cell, pbc), 2)
    package = comm.broadcast_string(package, root)
    rootdata, rootcell, rootpbc = pickle.loads(package)
    if rootdata is None or len(rootdata) == 0:
        raise ValueError("No data from 'root' atoms.  Empty atoms?!?")
    
    # Check for consistent cell and pbc arguments
    if cell is not None:
        if rootcell is None:
            raise TypeError("Cell given on another processor than the atoms.")
        if (cell.ravel() - rootcell.ravel()).max() > 1e-12:
            raise ValueError("Inconsistent cell specification.")
    else:
        cell = rootcell   # May still be None
    if pbc is not None:
        if rootpbc is None:
            raise TypeError("PBC given on another processor than the atoms.")
        if (pbc != rootpbc).any():
            raise ValueError("Inconsistent pbc specification.")
    else:
        pbc = rootpbc

    # Check for consistent atoms data
    if hasdata is not None:
        if hasdata != rootdata:
            raise ValueError("Atoms do not contain the sama data on different processors.")
    if "positions" not in rootdata:
        raise ValueError("Atoms do not have positions!")
    
    # Create empty atoms
    if atoms is None:
        atoms = ase.Atoms(cell=cell, pbc=pbc)
        for name in rootdata.keys():
            if name in atoms.arrays:
                assert np.sctype2char(atoms.arrays[name]) == rootdata[name][0]
                assert len(atoms.arrays[name]) == 0
            else:
                shape = (0,) + rootdata[name][1]
                atoms.arrays[name] = np.zeros(shape, rootdata[name][0])
        
    return ParallelAtoms(nCells, comm, atoms, cell=cell, pbc=pbc, 
                         distribute=distribute)



# A cleanup function should call MPI_Abort if python crashes to
# terminate the processes on the other nodes.
ase.parallel.register_parallel_cleanup_function()

# _oldexitfunc = getattr(sys, "exitfunc", None)
# def _asap_cleanup(lastexit = _oldexitfunc, sys=sys, time=time,
#                   comm = asap3.mpi.world):
#     error = getattr(sys, "last_type", None)
#     if error:
#         sys.stdout.flush()
#         sys.stderr.write("ASAP CLEANUP (node " + str(comm.rank) +
#                          "): " + str(error) +
#                          " occurred.  Calling MPI_Abort!\n")
#         sys.stderr.flush()
#         # Give other nodes a moment to crash by themselves (perhaps
#         # producing helpful error messages).
#         time.sleep(3)
#         comm.abort(42)
#     if lastexit:
#         lastexit()
# sys.exitfunc = _asap_cleanup
        
# END OF PARALLEL STUFF



# Script which tests the EMTv102.EMT() calculators get_potential_energy() and get_forces() function.
# The test is done by creating an atoms object consisting of two Cu atoms placed at a distance r from each other.
# The get_potential_energy() and get_forces() functions are called and the results are stored, then r is incremented 
# and the process is repeated. The Energy and Force of the system is then plottet as a function of r up to just 
# beyond the cutoff distance of approx 4.8. The numerical deriviative of the energy is then calculated and compared to the # force.

# Written by: Rasmus E. Christiansen

# Initialization
import numpy
from ase import EMT
import asap3
import matplotlib.pyplot as ppl
from ase import Atoms 

# The cutoff distance for cobber is defined, r_cut_Cu = 4.7695, rounded to 0 decimals
r_cut_Cu = 5
# The distance r is defined
rstart = r = 1.5
# The incrementation is defined
inc = 0.01
# The number of steps is defined
num_steps = int(round(r_cut_Cu/inc))

# The atoms object is created
atomsNew = Atoms(positions=[[0,0,0], [0,0,r]], symbols='CuCu', cell=(10,10,10))
atomsOld = Atoms(positions=[[0,0,0], [0,0,r]], symbols='CuCu', cell=(10,10,10))


# The calculator is initialized
BOHR = 0.5292

emt = asap3.EMT2013({29: {'eta2': 1.652 * 1/BOHR,'lambda': 1.906 * 1/BOHR,'kappa': 2.740 * 1/BOHR,'e0': -3.51,'V0': 2.476,'seq': 2.67*BOHR,'neq': 0.0091 * 1/(BOHR * BOHR * BOHR),'mass': 63.546}})

# The Energy is calculated using the Effective Medium Theory by attaching the EMT calculator to the atoms object.
atomsNew.set_calculator(emt)
atomsOld.set_calculator(EMT())


# Calculations

# The calculations for the forces and energy are performed
Enew = numpy.zeros([num_steps])
Eold = numpy.zeros([num_steps])
Fnew = numpy.zeros([num_steps,2,3])
Fold = numpy.zeros([num_steps,2,3])
for i in range(num_steps):
    # Energy and force are calculated and stored
    Enew[i] = atomsNew.get_potential_energy()
    Fnew[i,:] = atomsNew.get_forces()
    Eold[i] = atomsOld.get_potential_energy()
    Fold[i,:] = atomsOld.get_forces()
    # The distance r is incremented
    r += inc
    # The atoms object is updated
    atomsNew.set_positions([[0,0,0],[0,0,r]])
    atomsOld.set_positions([[0,0,0],[0,0,r]])
    


# Sets up the x-axis for the plots
x = rstart+inc*numpy.array(range(num_steps))

# Calculates the minimum for the energy
EminNew = min(Enew)
EminOld = min(Eold)
# Calculates the distance for the minimum energy
rminNew = rstart+inc*Enew.argmin()
rminOld = rstart+inc*Eold.argmin()


# Calculates the numerical deriviative of the energy in the z-direction for atom 1:
FnumNew = numpy.zeros([num_steps,2])
FnumOld = numpy.zeros([num_steps,2])
for i in range(num_steps-1):
    FnumNew[i] = [(Enew[i+1] - Enew[i]) / inc , -(Enew[i+1] - Enew[i]) / inc]
    FnumOld[i] = [(Eold[i+1] - Eold[i]) / inc , -(Eold[i+1] - Eold[i]) / inc]


# Plots
ppl.figure(1)
ppl.plot(x,Enew)
ppl.plot(x,Eold)
ppl.figure(2)
ppl.plot(x,Fnew[:,0,2])
ppl.plot(x,FnumNew[:,0])
ppl.plot(x,Fold[:,0,2])
ppl.plot(x,FnumOld[:,0])
ppl.figure(3)
ppl.plot(x,Fnew[:,1,2])
ppl.plot(x,FnumNew[:,1])
ppl.plot(x,Fold[:,1,2])
ppl.plot(x,FnumOld[:,1])
ppl.show()










from ase.units import kB, fs
from asap3 import EMT
from asap3.md.langevin import Langevin
from asap3.io import Trajectory
from ase.build import fcc111

from asap3.testtools import ReportTest
import numpy as np


for seed in [1, None]:
    positions = []
    for i_run in range(2):

        atoms = fcc111('Au', size=(3,3,3), vacuum=10.0)
        atoms.set_calculator(EMT())

        dyn = Langevin(atoms, timestep=5.0*fs, temperature=300*kB, friction=1e-1, seed=seed)
        dyn.run(1000)

        positions.append(atoms.get_positions())

    p1, p2 = positions
    is_pos_equal = np.allclose(p1, p2)
    print("Max difference:", abs(p1 - p2).max())
    if seed is None:
        ReportTest.BoolTest("Runs differ when not seeded", not is_pos_equal)
    else:
        ReportTest.BoolTest("Runs are equal when seeded", is_pos_equal)
ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from ase.md.verlet import VelocityVerlet
from ase.md.velocitydistribution import MaxwellBoltzmannDistribution
from ase.lattice.compounds import L1_2
from ase import data
from asap3.io.trajectory import *
from numpy import *
import sys, os, time
from asap3.testtools import ReportTest
from asap3.mpi import world

set_printoptions(threshold='nan')

cu = ase.data.atomic_numbers['Cu']
au = ase.data.atomic_numbers['Au']
cu3au_a = 3.72977

print_version(1)
#set_verbose(1)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

delete = True
dosim = True
# precision = 1e-8


def check_z(a):
    try:
        id = a.get_ids()
    except AttributeError:
        id = np.arange(len(a))
    z = a.get_atomic_numbers()
    z_expect = np.where(id % 4, cu, au)
    if ismaster:
        print(z[:11])
        print(z_expect[:11])
    assert (z == z_expect).all()

if ismaster:
    initial = L1_2(size=(15,15,15), symbol=(au,cu),
                   latticeconstant=cu3au_a, pbc=(1,0,0))
else:
    initial = None
if isparallel:
    atoms = MakeParallelAtoms(initial, cpulayout)
    print("Min ID", atoms.get_ids().min())
else:
    atoms = initial.copy()
  
check_z(atoms)

if dosim:
    print("Simulation: create the bundle")
    # Give a momentum distribution likely to cause migration
    MaxwellBoltzmannDistribution(atoms, 5000*units.kB)
    p = atoms.get_momenta()
    pz = p[10,2]
    p[:,2] += pz

    atoms.set_calculator(EMT())
    if isparallel:
        traj = BundleTrajectory("preservez.bundle", "w", atoms, split=True)
    else:
        traj = BundleTrajectory("preservez.bundle", "w", atoms)
    dyn = VelocityVerlet(atoms, 5*units.fs)
    dyn.attach(traj, interval=50)
    dyn.attach(check_z, interval=25, a=atoms)
    traj.write()
    dyn.run(150)
    traj.close()
else:
    print("Only reading, hope its there!")

print("Reading in serial mode:")
if dosim:
    traj = BundleTrajectory("preservez.bundle")
    for i, atoms in enumerate(traj):
        print("Task {0}, step {1}: found {2} atoms".format(world.rank, i, len(atoms)))
        check_z(atoms)
    traj.close()

world.barrier()
print("Reading in parallel mode.")
traj = BundleTrajectory("preservez.bundle")
for i in range(len(traj)):
    atoms = traj.get_atoms(i, cpulayout)
    print("Task {0}, step {1}: found {2} atoms".format(world.rank, i, len(atoms)))
    check_z(atoms)
traj.close()
del traj

world.barrier()
if delete:
    #if ismaster:
    print("Deleting trajectory")
    BundleTrajectory.delete_bundle("preservez.bundle")

print("Not crashed: test has passed!")



from __future__ import print_function
# This file provides information about which MPI implementation was
# used to compile the parallel version of Asap.  The make process
# copies this file to Swig/`arch` and adds a line with the path to mpicc.

import os

def printmpiinfo():
    bindir = os.path.dirname(mpicc)
    mpidir = os.path.dirname(bindir)
    print(mpidir)





import numpy as np

from asap3.MonteCarlo.Moves.Base import RandSphere

from ase.atoms import Atoms, symbols2numbers
from ase.data import covalent_radii

def RandomBoxAtoms(symbols, cell=[5.0, 5.0, 5.0]):
    numbers = symbols2numbers(symbols)
    scaled_positions = np.random.uniform(0.0, 1.0, (len(numbers), 3))
    return Atoms(numbers, scaled_positions=scaled_positions, cell=cell)

def RandomSphereAtoms(symbols, covalent=None, radius=None, vacuum=0.0):
    numbers = symbols2numbers(symbols)
    if covalent is None:
        covalent = 0.8 * np.mean(covalent_radii[numbers])
    if radius is None:
        radius = covalent * float(len(numbers))**(1.0/3.0)
    cell = 3.0 * radius * np.ones(3, float) + vacuum
    positions = radius * np.ones((len(numbers), 3))
    for i in range(len(numbers)):
        pos = radius * RandSphere(1) + 0.5 * cell
        while np.any(np.sum((positions - pos)**2, axis=1) < covalent**2):
            pos = radius * RandSphere(1) + 0.5 * cell
        positions[i] = pos.copy()
    #positions = radius * RandSphere(len(numbers)) + 0.5 * cell
    return Atoms(numbers, positions=positions, cell=cell)

def AlloyAtomsRandom(symbols, atoms):
    numbers_new = symbols2numbers(symbols)
    numbers_old = atoms.get_atomic_numbers()
    natoms = len(numbers_old)

    if len(numbers_old) != len(numbers_new):
        raise ValueError("The number of atoms in symbols (%i) " %(len(numbers_new),) +
                         "must be equal to that in atoms (%i)!" % (len(atoms),))

    numbers = []
    indexes = []
    for n in np.unique(numbers_new + numbers_old.tolist()):
        d = (numbers_new == n).sum() - (numbers_old == n).sum()
        if d > 0: # Add
            while d > 0:
                i = np.random.random_integers(0, len(numbers))
                numbers.insert(i, n)
                d -= 1
        elif d < 0: # Remove
            ind = np.arange(natoms)[numbers_old == n].tolist()
            while d < 0:
                i = np.random.random_integers(0, len(ind) - 1)
                indexes.append(ind.pop(i))
                d += 1

    numbers_old[indexes] = numbers
    atoms.set_atomic_numbers(numbers_old)
    return atoms



"""Usage: python test_reproducible.py [init]

Used to test if the EMT potential still produces the same energies, forces
and stresses as it used to.

Call it the first time with the "init" argument to generate a data file.

Subsequent calls calculate the same quantities, but compare them with the
database.
"""
from __future__ import print_function

from numpy import *
from asap3 import *
import pickle, os, sys, time

# PrintVersion(1)
if sys.version_info[0] >= 3:
    picklehack = {'encoding': 'latin1'}
else:
    picklehack = {}

def main():
    datafile = "test_reproducible2.dat"
    is_data = os.access(datafile, os.R_OK)
    initialize = 0
    if len(sys.argv) == 2 and sys.argv[1] == "init":
        initialize = 1
        if os.access(datafile, os.R_OK):
            print("ERROR: Data file already exists.  Delete it if you want to make a new one.")
            print("")
            print(__doc__)
            sys.exit(1)
    elif len(sys.argv) != 1:
        print("ERROR: Wrong command line.")
        print("")
        print(__doc__)
        sys.exit(1)
    elif not os.access(datafile, os.R_OK):
        print("ERROR: No data file.")
        print("")
        print(__doc__)
        return

    indata = pickle.load(open("test_reproducible.in", "rb"), **picklehack)
    lattice = Atoms(positions=indata['positions'], cell=indata['cell'], symbols=["Cu"]*len(indata['positions']), pbc=True)
    #_v = Verbose(1)
    # Make a lattice
    #lattice = FCCOrtho(((1,-1,0),(1,1,-2),(1,1,1)), (15,8,14), Copper,
    #                   symmetry=(1,1,1), half=1)
    print("Number of atoms", len(lattice))
    print(lattice.get_cell())
    # Perturb the positions
    r = lattice.get_positions()
    dr =  0.05 * sin(arange(3*len(lattice)))
    dr.shape = (-1,3)
    r += dr
    lattice.set_positions(r)

    atoms1 = Atoms(lattice)
    atoms1.set_calculator(EMT())

    print("Total energy:", atoms1.get_potential_energy())
    print("Stress:", atoms1.get_stress())
    #CNA(atoms1)
    #plot = atoms1.GetPlot()
    #time.sleep(10)

    z = lattice.get_atomic_numbers()
    z[0] = z[1] = 47
    lattice.set_atomic_numbers(z)
    atoms2 = Atoms(lattice)
    atoms2.set_calculator(EMT())
    
    if initialize:
        Cu = {}
        n = len(atoms1)
        Cu["energies"] = atoms1.get_potential_energies()
        Cu["forces"] = atoms1.get_forces()
        Cu["stresses"] = atoms1.GetStresses()
        assert Cu["energies"].shape == (n,)
        assert Cu["forces"].shape == (n, 3)
        assert Cu["stresses"].shape == (n, 6)
        AgCu = {}
        AgCu["energies"] = atoms2.GetPotentialEnergies()
        AgCu["forces"] = atoms2.GetCartesianForces()
        AgCu["stresses"] = atoms2.GetStresses()
        data = {"Cu": Cu, "AgCu": AgCu}
        pickle.dump(data, open(datafile, "wb"))
    else:
        data = pickle.load(open(datafile, "rb"), **picklehack)
        Cu = data["Cu"]
        AgCu = data["AgCu"]
        print("*** Checking pure copper ***")
        e = 0
        e = e + evaluate("energies", Cu["energies"], atoms1.get_potential_energies())
        e = e + evaluate("forces", Cu["forces"], atoms1.get_forces())
        e = e + evaluate("stresses", Cu["stresses"], atoms1.get_stresses())
        print("*** Checking silver in copper ***")
        e = e + evaluate("energies", AgCu["energies"], atoms2.get_potential_energies())
        e = e + evaluate("forces", AgCu["forces"], atoms2.get_forces())
        e = e + evaluate("stresses", AgCu["stresses"], atoms2.get_stresses())
        if e == 6:
            print("*** All tests passed ***")
        else:
            raise RuntimeError("*** THERE WERE ERRORS IN SOME TESTS! ***")

def evaluate(text, expected, actual):
    diff = max(abs(expected.flat[:] - actual.flat[:]))
    passed = diff < 1e-10
    if passed:
        print("Checking %s: max error = %g   OK" % (text, diff))
    else:
        print("Checking %s: max error = %g   FAILED!     <<<<<<<<<<" % (text, diff))
    #print "   average over all atoms:", sum(actual) / len(actual)
    return passed

    
main()



from __future__ import print_function
from asap3 import *
from numpy import *
from pickle import load, dump
from asap3.testtools import ReportTest
try:
    from ase.data import atomic_masses_legacy
except ImportError:
    atomic_masses_legacy = None
#from ase.md import VelocityVerlet
import sys

if atomic_masses_legacy is None:
    print("Skipping test - you are using an old ASE,")
    print("  making this test irrelevant.")
else:
    timeunit = 1.018047e-14             # Seconds
    femtosecond = 1e-15 / timeunit      # Marginally different from units.fs

    print_version(1)

    if sys.version_info[0] >= 3:
        picklehack = {'encoding': 'latin1'}
    else:
        picklehack = {}

    data = load(open("Verlet_legacymass.pickle", "rb"), **picklehack)
    init_pos = array(data["initial"])
    init_pos.shape = (-1,3)
    init_box = array(data["box"])
    init_box.shape = (3,3)
    atoms = Atoms(positions=init_pos, cell=init_box)
    atoms.set_atomic_numbers(47*ones((len(atoms),)))
    atoms.set_masses(atomic_masses_legacy[atoms.numbers])
    print(atoms.get_masses()[0], atoms.get_masses().shape)
    atoms.set_calculator(EMT())

    dyn = VelocityVerlet(atoms, 2 * femtosecond)
    dyn.attach(MDLogger(dyn, atoms, '-', peratom=True), interval=5)

    etot = None

    for i in range(10):
        dyn.run(20)
        epot = atoms.get_potential_energy() / len(atoms)
        ekin = atoms.get_kinetic_energy() / len(atoms)
        if etot is None:
            etot = epot + ekin
        else:
            ReportTest("Energy conservation:", epot + ekin, etot, 0.001)

    final_pos = array(data["final"])
    diff = max(abs(atoms.get_positions().flat - final_pos))
    print("Maximal deviation of positions:", diff)
    ReportTest("Maximal deviation of positions", diff, 0, 1e-9)

    diff = max(abs(atoms.get_stresses().flat - array(data["stress"])))

    print("Maximal deviation of stresses:", diff)
    ReportTest("Maximal deviation of stresses", diff, 0, 1e-9)
    ReportTest.Summary()




# Reading old ASAP files (version 1.X NetCDF files).

"""Asap.IO.ReadOldFiles reads NetCDF files from ASAP version 1.X.

The following functions are defined:

`Reader(filename)`:
An object for reading old-style (ASAP 1.x) NetCDF files.

`Reader.Read(frame = None)`:
Read a given frame into an ASAP ListOfAtoms object.  The
default frame is the one specifies by .Goto or in the
constructor.
   
`Read(filename, frame=-1)`:
    Reads a NetCDF file containing a simulation, returning a
    ListOfAtoms object.  Mainly for serial simulations.

"""

__docformat__ = "restructuredtext en"

import Asap as _Asap
from Scientific.IO.NetCDF import NetCDFFile
import Numeric

class Reader:
    """Reads configurations from a NetCDF file (ASAP 1.x format).

Read configuration number "N" from a NetCDF file:    
>>> atoms = Reader("stuff.nc").Read(N)

Read last configuration from a NetCDF file:
>>> atoms = Reader("stuff.nc").Read()

Read several configurations:
>>> r = Reader("things,.nc")
>>> atoms1 = r.Read(0)
>>> atoms2 = r.Read(1)
"""
    def __init__(self, filename, whatToRead = None):
        """Construct a reader from a filename."""        
        self.whatToRead = whatToRead
        try:
            self.nc = NetCDFFile(filename, 'r')
        except IOError:
            self.nc = NetCDFFile(filename + ".nc", 'r')
        
    def __del__(self):
        try:
            self.nc.close()
        except IOError:
            pass

    def Close(self):
        """Close the associated NetCDF file."""
        self.nc.close()

    def GetNetCDFFile(self):
        """Get the associated NetCDF file handle."""
        return self.nc

    def Read(self, frame=-1):
        """Reads a frame, returning a ListOfAtoms.

        Per default it reads the last frame, but other frames can be
        specified.
        """
        vars = self.nc.variables
        positions = vars["cartesianPositions"][frame]
        cell = vars["basisVectors"][frame]
        peri = vars["periodic"][:]
        atoms = _Asap.ListOfAtoms(positions=positions, cell=cell,
                                  periodic=peri)
        if self.whatToRead is None:
            self.whatToRead = []
            for stuff in ["CartesianMomenta", "Classes", "AtomicNumbers"]:
                if stuff[0].lower() + stuff[1:] in vars:
                    self.whatToRead.append(stuff)
        for stuff in self.whatToRead:
            if stuff == "CartesianMomenta":
                atoms.SetCartesianMomenta(vars["cartesianMomenta"][frame].astype("d"))
            elif stuff == "Classes":
                atoms.SetTags(vars["classes"][frame])
            elif stuff == "AtomicNumbers":
                if len(vars["atomicNumbers"]) == 1:
                    atoms.SetAtomicNumbers(vars["atomicNumbers"][0] +
                                           Numeric.zeros(len(atoms)))
                else:
                    atoms.SetAtomicNumbers(vars["atomicNumbers"][:])
            else:
                raise RuntimeError("Don't know how to read " + stuff)
        return atoms

def Read(filename, n = -1, optional = None):
    return Reader(filename, optional).Read(n)



from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest
from ase.build import bulk
import numpy as np
import itertools

openkimmodel = 'EMT_Asap_Standard_JacobsenStoltzeNorskov_1996_%s__%s'

shortkimid = {
    'Ag': 'MO_303974873468_001',
    'Al': 'MO_623376124862_001',
    'Au': 'MO_017524376569_001',
    'Cu': 'MO_396616545191_001',
    'Ni': 'MO_108408461881_001',
    'Pd': 'MO_066802556726_001',
    'Pt': 'MO_637493005914_001',
    }

#set_verbose(1)
step = 500


if OpenKIMsupported:
    pbc_list = itertools.cycle(((1,1,1), (0,0,0), (0,0,1)))
    size_list = itertools.cycle(((10, 10, 10), (20, 5, 2), (2, 2, 2), (1, 1, 1)))
    elements = ('Al', 'Ag', 'Au', 'Cu', 'Ni', 'Pd', 'Pt')
else:
    elements = ()
    print("OpenKIM support is not compiled into Asap.")

for element in elements:
    for i in (0, 1):
        pbc = next(pbc_list)
        size = next(size_list)
        txt = ("%s=%i%i%i-%i-%i-%i " % ((element,) + pbc + size))
        # Test that EMT reimported through OpenKIM gives the right results.
        atoms_kim = bulk(element).repeat(size)
        #atoms_kim = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
        #                    size=(30, 30, 30),
        #                    symbol="Cu")
        natoms = len(atoms_kim)
        atoms_kim.set_pbc(pbc)
        r = atoms_kim.get_positions()
        r.flat[:] += 0.1 * np.sin(np.arange(3*natoms))
        atoms_kim.set_positions(r)
        atoms_emt = atoms_kim.copy()
        try:
            kim = OpenKIMcalculator(openkimmodel % (element, shortkimid[element]))
        except AsapError as oops:
           if oops.args[0].startswith('Failed to initialize OpenKIM model'):
               print("OpenKIM model {} not installed - skipping test.".format(
                   openkimmodel % (element, shortkimid[element])))
               continue
           else:
               raise   # Something else went wrong.
        emt = EMT()
        emt.set_subtractE0(False)
        atoms_kim.set_calculator(kim)
        atoms_emt.set_calculator(emt)
        ek = atoms_kim.get_potential_energy()
        ee = atoms_emt.get_potential_energy()
        ReportTest(txt+"Total energy", ek, ee, 1e-8)
        ek = atoms_kim.get_potential_energies()
        ee = atoms_emt.get_potential_energies()
        for i in range(0, natoms, step):
            ReportTest(txt+"Energy of atom %i" % (i,), ek[i], ee[i], 1e-8)
        fk = atoms_kim.get_forces()
        fe = atoms_emt.get_forces()
        n = 0
        for i in range(0, natoms, step):
            n = (n + 1) % 3
            ReportTest(txt+"Force(%i) of atom %i" % (n, i), fk[i, n], fe[i, n], 1e-8)
        sk = atoms_kim.get_stress()
        se = atoms_emt.get_stress()
        for i in range(6):
            ReportTest(txt+"Stress(%i)" % (i,), sk[i], se[i], 1e-8)
        sk = atoms_kim.get_stresses()
        se = atoms_emt.get_stresses()
        for i in range(0, natoms, step):
            n = (n + 1) % 6
            # Volume per atom is not defined the same way: greater tolerance needed
            ReportTest(txt+"Stress(%i) of atom %i" % (n, i), sk[i, n], se[i, n], 1e-3)
else:
    print("OpenKIM support is not compiled into Asap.")

    
ReportTest.Summary()



#!/usr/bin/env python

from __future__ import print_function
from numpy import *
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.setup.dislocation import Dislocation
from asap3.analysis import CNA
from ase.visualize.primiplotter import *
from asap3.testtools import ReportTest


print_version(1)

splitting = 5
#size = (50, 88, 35)
size = (30, 25, 7)

Gold = "Au"
slab = FaceCenteredCubic(directions=((1,1,-2), (-1,1,0), (1,1,1)),
                         size=size, symbol=Gold)
basis = slab.get_cell()
print(basis)
print("Number of atoms:", len(slab))

center = 0.5 * array([basis[0,0], basis[1,1], basis[2,2]]) + array([0.1, 0.1, 0.1])
offset = 0.5 * splitting * slab.miller_to_direction((-1,0,1))
print(center)

d1 = Dislocation(center - offset, slab.miller_to_direction((-1,-1,0)),
                 slab.miller_to_direction((-2,-1,1))/6.0)
d2 = Dislocation(center + offset, slab.miller_to_direction((1,1,0)),
                 slab.miller_to_direction((1,2,1))/6.0)

atoms = Atoms(slab)
(d1+d2).apply_to(atoms)
del slab

print("Now running CNA")

atoms.set_calculator(EMT())
c = CNA(atoms)
atoms.set_tags(c)

expected = [27703, 314, 3483]

for i in range(3):
    print(i, sum(equal(c, i)))
    ReportTest("Number of atoms with CNA class "+str(i), sum(equal(c, i)), expected[i], 0)
    
ReportTest.Summary()




"""MaterialPropertiesData - module to handle material property data

The module loads the metarial data form one or more files with the format:

    Element(s);Structure;Property;Value;Temp;Reference
    Al;fcc;Ecoh;3.39;0;C. Kittel, Introduction to solid state physics, ...
    Al,Ag;l12;Eheat;-0.08708;0;Jakob Schi\otz 2013

Property names and units::

a, c
    Lattice constants

B
    Bulk modulus (GPa)

A
    Cubic anisotopy ratio = 2*C44/(C11-C12) (GPa)

C11, C12, C44
    Cubic elastic constants (GPa)
    
C11, C12, C13, C33, C44
    Hexagonal elastic constants (GPa)
    
Ecoh
    Cohesive energy (eV)  
    
Evac
    Vacancy formation energy (eV)

Eheat
    Alloy heat of formation (eV)

E100, E111
    Surface energies of 100, 111 (etc) surfaces (eV/atom)

Eisf
    Intrinsic stacking fault energy (J/m^2)

Ehcpfcc
    Energy difference between HCP and FCC structure, both with
    their optimal lattice constants

Ebccfcc
    Similar, but BCC minus FCC energy.

"""
from __future__ import print_function

from ase.data import atomic_numbers, reference_states
from ase.utils import basestring

class MaterialPropertiesData:
    def __init__(self, filenames):
        self.data = {}
        if isinstance(filenames, basestring):
            filenames = [filenames,]
        for file in filenames:
            self.read(file)

    def get(self, elements, property, structure=None, temperature=None,
            calculate=False, reference=False):
        if isinstance(elements, basestring):
            elements = (elements,)
        elif isinstance(elements, list):
            elements = tuple(elements)

        if structure == None:
            if len(elements) == 1:
                Z = atomic_numbers[elements[0]]
                structure = reference_states[Z]['symmetry']
            else:
                raise ValueError('Structure must be specified for an alloy.')

        if elements not in self.data:
            if len(elements) == 1 and property in ['a', 'c', 'ca']:
                Z = atomic_numbers[elements[0]]
                assert structure == reference_states[Z]['symmetry']
                a = reference_states[Z]['a']
                if property == 'c':
                    return a * reference_states[Z]['c/a']
                elif property == 'ca':
                    return reference_states[Z]['c/a']
                else:
                    return a
            else:
                raise ValueError('No data for the element(s): %s.' % (elements,))

        key = structure + '_' + property
        data = self.data[elements]
        if key not in data or calculate:
            if property == 'B':
                value = self.calc_B(elements, structure, temperature)
            elif property == 'A':
                value = self.calc_A(elements, structure, temperature)
            elif property == 'ca':
                return self.get(elements, 'c', structure, temperature) / self.get(elements, 'a', structure, temperature)
            else:
                raise ValueError('Cannot find or calculate the property %s %s %s.' % (key, property, str(elements)))
        else:
            found = False
            if temperature != None:
                for (val, temp, ref) in data[key]:
                    if temp == temperature:
                        found = True
                        break
            if not found:
                (val, temp, ref) = data[key][0]

            if reference:
                value = ref
            else:
                value = val

        return value

    def calc_B(self, elements, structure=None, temperature=None):
        C11 = self.get(elements, 'C11', structure, temperature)
        C12 = self.get(elements, 'C12', structure, temperature)
        return 1.0/3.0*(C11 + 2*C12)

    def calc_A(self, elements, structure=None, temperature=None):
        C11 = self.get(elements, 'C11', structure, temperature)
        C12 = self.get(elements, 'C12', structure, temperature)
        C44 = self.get(elements, 'C44', structure, temperature)
        return 2.0*C44/(C11 - C12)

    def read(self, filename):
        f = open(filename)
        for line in f.readlines():
            line = line[:-1].split(';') #'\t')

            elements = tuple(line[0].split(','))
            if elements[0] not in atomic_numbers:
                continue

            if elements not in self.data:
                self.data[elements] = {}

            property = '_'.join(line[1:3])
            if property not in self.data[elements]:
                self.data[elements][property] = []

            data = (float(line[3]), int(line[4]), line[5])
            self.data[elements][property].append(data)
        f.close()

    def write(self, filename):
        f = open(filename, 'w')
        f.write('Element(s);Structure;Property;Value;Temperature;Reference\n')
        for elements in self.data.keys():
            for property in self.data[elements].keys():
                for data in self.data[elements][property]:
                    data = elements + tuple(property.split('_')) + data
                    if len(elements) > 1:
                        format = '%s,%s;%s;%s;%f;%i;%s\n'
                    else:
                        format = '%s;%s;%s;%f;%i;%s\n'
                    f.write(format % tuple(data))
        f.close()

if __name__ == '__main__':
    print('Testing...')
    mp_data = MaterialPropertiesData('material_properties.csv')
    print(mp_data.get('Pt', 'Ecoh'))




"""Get EMT2013 parameters from fitting script output file."""

from ase.atoms import string2symbols

def get_parameters(file, number=2):
    file = open(file)
    text = file.read()
    file.close()

    # Find elements
    s = -1
    for i in range(number):
        s = text.find('Optimization', s + 1)
        if s < 0:
            raise ValueError("No results in file (keyword Optimization missing in file)")
    e = text.find('\n', s)
    errfunc = float(text[s:e].split()[6])
    result = {}

    s = e + 1
    e = text.find('Fitting values', s) - 4
    for line in text[s:e].split('\n'):
        words = line.strip().split()
        if words[1] == 'parameters':
            elements = tuple(string2symbols(words[0]))
            result[elements] = []
        else:
            result[elements].append(float(words[1]))
    
    return result, errfunc



from __future__ import print_function
# Test that various observers and analysis tools do not interfere with
# the energy calculator e.g. by messing up the neighbor list.

from asap3 import *
from asap3.md.verlet import VelocityVerlet
from asap3.md.langevin import Langevin
from ase.lattice.cubic import *
from asap3.analysis import RestrictedCNA, RadialDistributionFunction
from numpy import *
from asap3.testtools import ReportTest

def Compare(name, atoms, observer, ref, interval=4):
    dyn = VelocityVerlet(atoms, 5*units.fs)
    dyn.attach(observer, interval=interval)
    r = []
    for i in range(50):
        dyn.run(5)
        epot = atoms.get_potential_energy() / len(atoms)
        ekin = atoms.get_kinetic_energy() / len(atoms)
        r.append([epot, ekin, epot+ekin])
    r = array(r)
    diff = r - ref
    maxdiff = diff.max()
    print("Maximal difference is", maxdiff)
    ReportTest(name, maxdiff, 0.0, 1e-6)
    
print_version(1)

print("Making initial system")
iniatoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                             size=(10,5,5), symbol="Cu", pbc=(1,1,0))
iniatoms.set_calculator(EMT())
inidyn = Langevin(iniatoms, 5*units.fs, 450*units.kB, 0.05)
inidyn.run(100)
print("Temperature is now", iniatoms.get_kinetic_energy() / (1.5*units.kB*len(iniatoms)), "K")

print("Making reference simulation")
refatoms = Atoms(iniatoms)
refatoms.set_calculator(EMT())
dyn = VelocityVerlet(refatoms, 5*units.fs)
ref = []
for i in range(50):
    dyn.run(5)
    epot = refatoms.get_potential_energy() / len(refatoms)
    ekin = refatoms.get_kinetic_energy() / len(refatoms)
    ref.append([epot, ekin, epot+ekin])

ref = array(ref)

print("Testing RestrictedCNA")
atoms = Atoms(iniatoms)
atoms.set_calculator(EMT())
cna = RestrictedCNA(atoms)
Compare("RestrictedCNA", atoms, cna.analyze, ref)

print("Testing RadialDistributionFunction")
atoms = Atoms(iniatoms)
atoms.set_calculator(EMT())
rdf = RadialDistributionFunction(atoms, 4.0, 25)
Compare("RadialDistributionFunction", atoms, rdf.update, ref)




"""Python module for running PTM on an Asap list of atoms.

This module runs Polyhedral Template Matching on an Atoms object from ASE/Asap.
It requires that both ASE and Asap are installed.

This module can also be used as inspiration for how to use Polyhedral Template
Matching in other codes.

"""

from __future__ import print_function
import asap3
from asap3.Internal.Subject import Subject
import numpy as np
import sys

def PTM(atoms, cutoff=10.0, rmsd_max=None, **kwargs):
    """Run Complex Hull Analysis on an Atoms object.

    Parameters:
    atoms: The atoms object

    cutoff: A cutoff used for the neighborlist.  Must be large enough that all
        nearest neighbors are returned (second-nearest for BCC).  Using a too
        large value may impact performance, but not results.
        
    rmsd_max=None: If set, matches with a RMSD above this threshold are 
        reclassified as unknown structure (type 0).
        
        This is done as post-processing, and is completely equivalent to
            data['structure'][data['rmsd'] > rmsd_max] = 0

    target_structures=None: A tuple of structures to be investigated.
        It defaults to ('sc', 'fcc', 'hcp', 'ico', 'bcc').
        It MUST be a tuple, not a list or other sequence (bug?).
    
    calculate_strains=False: Set to True to calculate strains.

    quick=False: Set to True to skip topological ordering of neighbors.
        It gives a factor of two in speed, but more misclassified atoms at
        high temperatures.

    return_nblist=False:
        Return the NeighborList object used internally. This may be used for
        preprocessing. Do not enable unless you need it, as this object can
        use a lot of memory.

    return_mappings=None:
        Return the mapping of neighbors to the template used internally.
        Set to True to get mappings for all structures, or a tuple of
        structures to get for some (see target_structures).
         
    Returns:
    A dictionary with NumPy arrays as values.
    Each of these are a NumPy array of the same length as the number of atoms.
    
    Key names and descriptions of returned data.  In the description, i is the
    first index of the array and identifies the atom.
    
    'structure': The local crystal structure around atom i, if any.
        0 = none; 1 = SC; 2 = FCC; 3 = HCP; 4 = Icosahedral; 5 = BCC.

    'alloytype': The alloy structure identified.
        0 = unidentified; 1 = pure element; 2 = L1_0;
        3 = L1_2 majority atom; 4 = L1_2 minority atom.
        (0 is returned if structure != 2 or if no known alloy structure
        is recognized)

    'rmsd': The RMSD error in the fitting to the template, or INF if
        no structure was identified.

    'scale': The average distance to the nearest neighbors for
        structures 1-4; or the average distance to nearest and
        next-nearest neighbors for structure 5 (BCC); or INF if no
        structure was identified.

    'orientation': The orientation of the crystal lattice, expressed as a
        unit quaternion.  If no structure was found, the illegal
        value (0, 0, 0, 0) is returned.

    'strain' (only present if calculate_strains=True).  The strain
        tensor as a symmetric 3x3 matrix.  The trace of the matrix is
        1.0, since a hydrostatic component of the strain cannot be
        determined without a-priori knowledge of the reference
        lattice parameter.  If such knowledge is available, the
        hydrostatic component of the strain can be calculated from
        scales[i].
         
    'info': A tuple of two integers, the number of atoms analyzed by PTM
        and the number of atoms skipped due to insufficient neighbors.

    'nblist': (only if return_nblist is True) A neighbor list object.

    """
    for argument in ('target_structures', 'return_mappings'):
        if argument in kwargs:
            # The C++ code only accepts tuples of bytes (str in Python 2),
            # we need more flexibility.
            structs = kwargs[argument]
            if structs is None:
                continue
            if sys.version_info[0] == 3:
                unic = str
            else:
                unic = unicode
            if isinstance(structs, tuple):
                structs = list(structs)  # Make mutable
            for i, s in enumerate(structs):
                if isinstance(s, unic):
                    structs[i] = s.encode()  # Convert to bytes
            kwargs[argument] = tuple(structs)
    data = asap3._asap.PTM_allatoms(atoms, cutoff, **kwargs)
    if rmsd_max:
        data['structure'][data['rmsd'] > rmsd_max] = 0
    return data

class PTMobserver(Subject):
    """Polyhedral Template Matching observer object.

    Per default, sets the tags of the atoms according to their local crystal
    structure.  0 = none; 1 = FCC; 2 = HCP; 3 = BCC; 4 = Icosahedral; 5 = SC.
        
    An optional argument `quantity` can be set to 'alloytype' or None, then
    the tags are set to the alloytype calculated by the PTM function (see 
    that function's documentation); or not set if None is given.
    
    This class is intended to use as an observer, so its analyze()
    function is called automatically by e.g. the dynamics.  It can
    itself act as a subject, so a Plotter or a Trajectory can be
    called just after the calculations.
    
    It has a method get_data which returns the full dictionary of PTM
    data from the last calculation.

    Parameters:
    atoms: The atoms object

    cutoff: A cutoff used for the neighborlist.  Must be large enough that all
        nearest neighbors are returned (second-nearest for BCC).  Using a too
        large value may impact performance, but not results.
        
    rmsd_max=None: If set, matches with a RMSD above this threshold are 
        reclassified as unknown structure (type 0).
        
        This is done as post-processing, and is completely equivalent to
            data['structure'][data['rmsd'] > rmsd_max] = 0

    target_structures=None: A tuple of structures to be investigated.
        It defaults to ('sc', 'fcc', 'hcp', 'ico', 'bcc').
        It MUST be a tuple, not a list or other sequence (bug?).
    
    calculate_strains=False: Set to True to calculate strains.

    quick=False: Set to True to skip topological ordering of neighbors.
        It gives a factor of two in speed, but more misclassified atoms at
        high temperatures.

    quantity='structure': The quantity used to set the tags.
    
    analyze_first=True: Should an analysis be made as soon as
        this object is created?  Leave as true if you plan to attach a
        Trajectory as an observer to this object, and if the
        Trajectory will save the initial state.
    """
    def __init__(self, atoms, cutoff=10.0, rmsd_max=None, target_structures=None, 
                 calculate_strains=False, quick=False, quantity='structure',
                 analyze_first=True):
        Subject.__init__(self)
        self.atoms = atoms
        self.cutoff = cutoff
        self.rmsd_max = rmsd_max
        self.target_structures = target_structures
        self.strains = calculate_strains
        self.quick = quick
        self.quantity = quantity
        if analyze_first:
            self.analyze()  # There will not be any observers yet.
        
    def analyze(self):
        "Runs the PTM analysis."
        self.data = PTM(self.atoms, cutoff=self.cutoff, rmsd_max=self.rmsd_max, 
                        target_structures=self.target_structures,
                        calculate_strains=self.strains,
                        quick=self.quick)
        if self.quantity:
            self.atoms.set_tags(self.data[self.quantity])
        self.call_observers()
        
    update = analyze
    
    def get_data(self):
        return self.data
    
    
class PTMdislocations(PTMobserver):
    """Polyhedral Template Matching observer object.

    Sets the tags of the atoms according to their local crystal
    structure, after postprocessing of the HCP atoms for
    dislocation detection.  The dislocation detection assumes
    that the majority of the system is in the FCC structure.

    Tags are set to 0 = none; 1 = FCC; 2 = HCP; 3 = BCC;
    4 = Icosahedral; 5 = SC; 6 = Twin boundary;
    7 = Stacking fault; 8 = Partial dislocation line.
   
    Otherwise, this object is like the PTMobserver, refer
    to it for further documentation and parameters (with the
    exception that the ``structure`` parameter is ignored).
    """
    # For HCP atoms, which neighbors are in the same basal plane, and
    # which are in the planes above and below?  The neighbors are located
    # like this.
    #
    #  (x) Basal plane atom         [y] Atom in plane above
    #
    #                      (7)
    #
    #
    #           (11)  [5]             (8)
    #
    #
    #                      (@)   [3]
    #
    #
    #           (12)  [1]             (9)
    #
    #
    #                      (10)
    #
    #   6 below 5; 4 below 3; 2 below 1.
    #   In this numbering, the central atom(@) is number 0, but that is not returned
    #   by the PTM code, so all indices must be one less than the number above.
    hcp_inplane = np.array((7, 8, 9, 10, 12, 11)) - 1
    hcp_above = np.array((5, 3, 1)) - 1
    hcp_below = np.array((6, 4, 2)) - 1
    def analyze(self):
        "Runs the PTM analysis."
        self.data = PTM(self.atoms, cutoff=self.cutoff, rmsd_max=self.rmsd_max, 
                        target_structures=self.target_structures,
                        calculate_strains=self.strains,
                        quick=self.quick, return_mappings=('hcp',))
        new_s = self.data['structure']
        s = np.array(new_s)  # A copy so we can change the original.
        mappings = self.data['mappings']
        hcpatoms = (s == 2).nonzero()[0]
        if getattr(self.atoms, "parallel", False):
            # Parallel simulation - we need to communicate the structures
            nghosts = len(self.atoms.ghosts['positions'])
            label = '_ptm_disloc_analysis_internal'
            self.atoms.arrays[label] = s
            self.atoms.ghosts[label] = np.zeros(nghosts, dtype=s.dtype)
            del s # Suppress a warning from asap - no extra references are allowed
            self.atoms.get_calculator().update_ghost_data(self.atoms)
            s = self.atoms.arrays[label]  # Restore s - same as before.
            ghost_structs = self.atoms.ghosts[label]
            del self.atoms.ghosts[label]
            del self.atoms.arrays[label]
            s = np.concatenate((s, ghost_structs))
        for i in hcpatoms:
            mapping = mappings[i]
            nhcp = 0
            nfcc = 0
            nbsum = np.zeros(3)
            # Atom types of the neighbors
            inplane = s[mapping[self.hcp_inplane]] == 2
            inplanesum = inplane.sum()
            above = s[mapping[self.hcp_above]] == 2
            abovesum = above.sum()
            below = s[mapping[self.hcp_below]] == 2
            belowsum = below.sum()
            fccsum = (s[mapping] == 1).sum()
            # Heuristic for finding typical structures of HCP atoms
            if inplanesum == 6 and abovesum == 0 and belowsum == 0:
                # A single basal plane of HCP atoms: Twin boundary
                new_s[i] = 6
            elif inplanesum == 6 and ((abovesum == 3 and belowsum == 0) or
                                      (abovesum == 0 and belowsum == 3)):
                # Looks like two parallel planes of HCP atoms: Stacking fault.
                new_s[i] = 7
            elif (3 <= inplanesum <= 5
                  and inplanesum + abovesum + belowsum + fccsum >= 11
                  and (bool(abovesum) ^ bool(belowsum))):
                # A partial basal plane, with a possibly partial plane either
                # above or below, but not both. At most one non-HCP/FCC atom.
                #
                # This could be the edge of a SF ribbon (i.e. a partial dislocation
                # core) if the atoms in the plane are on the same side.  Check that
                # there are only two boundaries between HCP and non-HCP atoms
                rolled = np.roll(inplane, 1)
                if np.logical_xor(inplane, rolled).sum() == 2:
                    new_s[i] = 8
        # Kill the mappings dictionary to save memory
        del mappings, self.data['mappings']
        self.atoms.set_tags(new_s)
        self.atoms.set_array("rmsd", self.data['rmsd'])
        self.call_observers()
        
    update = analyze



from __future__ import print_function
from asap3 import *
from asap3.md.verlet import VelocityVerlet
from ase.lattice.cubic import FaceCenteredCubic
from asap3.io.trajectory import *
#from ase.io.bundletrajectory import BundleTrajectory
from ase.io import write
from numpy import *
import sys, os, time
from asap3.testtools import ReportTest

#DebugOutput("output.%d")
print_version(1)
delete = True
#precision = 1.5e-4  # We store positions in single precision.
precision = 1e-8

def maketraj(atoms, t, nstep):
    e = [atoms.get_potential_energy()]
    dyn = VelocityVerlet(atoms, 5*units.fs)
    for i in range(nstep):
        dyn.run(10)
        e.append(atoms.get_potential_energy())
        if t is not None:
            t.write()
    return e

def maketraj_no_e(atoms, t, nstep):
    dyn = VelocityVerlet(atoms, 5*units.fs)
    for i in range(nstep):
        dyn.run(10)
        if t is not None:
            t.write()

def checktraj(t, e, recalc=True):
    i = 0
    for atoms in t:
        if recalc:
            atoms.set_calculator(EMT())
        ReportTest("Checking frame %d" % (i,), atoms.get_potential_energy(),
                   e[i], precision)
        i += 1

initial = FaceCenteredCubic(size=(10,10,10), symbol="Cu", pbc=(1,0,0))
initial.set_momenta(zeros((len(initial),3)))
initial.set_tags(arange(len(initial)))

atoms = initial.copy()
atoms.set_calculator(EMT())
print("Writing trajectory")
traj = BundleTrajectory("traj1.bundle", "w", atoms, singleprecision=False)
traj.write()
energies = maketraj(atoms, traj, 10)
traj.close()

print("Reading trajectory (recalculating energies)")
traj = BundleTrajectory("traj1.bundle")
checktraj(traj, energies)

print("Reading trajectory (stored energies)")
traj = BundleTrajectory("traj1.bundle")
checktraj(traj, energies, recalc=False)

print("Repeating simulation")
atoms = traj[5]
atoms.set_calculator(EMT())
energies2 = maketraj(atoms, None, 5)
for i in range(5):
    ReportTest("Rerun[%d]" % (i,), energies2[i], energies[i+5], precision)
traj.close()

print("Appending to trajectory")
atoms = BundleTrajectory("traj1.bundle")[-1]
atoms.set_calculator(EMT())
traj = BundleTrajectory("traj1.bundle", "a", atoms)
energies2 = maketraj(atoms, traj, 5)
traj.close()

print("Reading longer trajectory")
traj = BundleTrajectory("traj1.bundle")
checktraj(traj, energies + energies2[1:])

print("Writing trajectory with write")
write("traj2.bundle", atoms, format='bundletrajectory')

print("Writing a trajectory without precalculated energies.")
atoms = initial.copy()
atoms.set_calculator(EMT())
traj = BundleTrajectory("traj3.bundle", "w", atoms, singleprecision=False)
traj.write()
maketraj_no_e(atoms, traj, 10)
traj.close()
traj = BundleTrajectory("traj3.bundle")
checktraj(traj, energies, recalc=False)

if delete:
    print("Deleting trajectory")
    BundleTrajectory.delete_bundle("traj1.bundle")
    BundleTrajectory.delete_bundle("traj2.bundle")
    BundleTrajectory.delete_bundle("traj3.bundle")

    
ReportTest.Summary()



from __future__ import print_function
# Testing various potentials in parallel simulations

from asap3 import *
from asap3.mpi import world
from asap3.testtools import ReportTest
from asap3.GuptaParameters import PtY_parameters as PtY_Gupta
from asap3.EMT2013Parameters import sihb_PtY_parameters as PtY_EMT2013
from ase.lattice.cubic import FaceCenteredCubic
from ase.lattice.compounds import L1_2
from ase.visualize import view
from ase.data import atomic_numbers, reference_states

import os
import pickle
import time
import numpy as np

keep = 50   # Keep every 50 data point

ismaster = world.rank == 0
isparallel = world.size > 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]
else:
    raise RuntimeError("Not able to run on %i CPUs." % (world.size,))

# This may be run with Test or Test/parallel as working directory.
selfname = 'parallelPotentials.py'
resultname = 'parallelPotentials.pickle'
if not selfname in os.listdir('.'):
    if 'parallel' in os.listdir('.') and selfname in os.listdir('parallel'):
        resultname = os.path.join('parallel', resultname)
    else:
        raise RuntimeError("Cannot find " + selfname)
   
firsttime = (len(sys.argv) >= 2 and sys.argv[1] == '--first') 
if firsttime and os.path.exists(resultname):
    print("This will overwrite the result file '%s'." % (resultname,))
    print("If you don't want to do that press Ctrl-C within 10 seconds.")
    time.sleep(10)
    print("Proceeding...")
elif not firsttime and not os.path.exists(resultname):
    print("'%s' not found!" % (resultname,))
    print("Maybe you need to create it with the --first option.")
    sys.exit(1)

if firsttime:
    if isparallel:
        print("Cannot create result file (--first option) when running in parallel.")
        sys.exit(1)
    result = None
    newresult = {}
else:
    try:
        result = pickle.load(open(resultname, "rb"))
    except UnicodeDecodeError:
        result = pickle.load(open(resultname, "rb"), encoding='latin1')
        
def myfcc(symbol, pbc):
    "Make an fcc lattice with standard lattice constant"
    if ismaster:
        a = FaceCenteredCubic(symbol=symbol, size=(10,10,10), pbc=pbc)    
        dx = 0.01 * np.sin(0.1 * np.arange(len(a) * 3))
        dx.shape = (len(a),3)
        a.set_positions(a.get_positions() + dx)
        a.set_momenta(np.zeros((len(a),3)))
        #view(a)
    else:
        a = None
    if isparallel:
        a = MakeParallelAtoms(a, cpulayout)
    return a

def myalloy(symbols, pbc):
    "Create an L1_2 alloy (AB3)."
    a1 = reference_states[atomic_numbers[symbols[0]]]['a']
    a2 = reference_states[atomic_numbers[symbols[1]]]['a']
    a0 = (a1 + 3*a2)/4
    if ismaster:
        a = L1_2(symbol=symbols, size=(10,10,10), latticeconstant=a0, pbc=pbc)    
        dx = 0.01 * np.sin(0.1 * np.arange(len(a) * 3))
        dx.shape = (len(a),3)
        a.set_positions(a.get_positions() + dx)
        a.set_momenta(np.zeros((len(a),3)))
        #view(a)
    else:
        a = None
    if isparallel:
        a = MakeParallelAtoms(a, cpulayout)
    return a
    
    
def test(atoms, name, dostress=True):
    energy = atoms.get_potential_energy()
    forces = atoms.get_forces()
    positions = atoms.get_positions()
    if dostress:
        stresses = atoms.get_stresses()
        stress = atoms.get_stress()
    if isparallel:
        ids = atoms.get_ids()
    else:
        ids = np.arange(len(atoms))
    if result:
        # Test that we still get the same
        data = result[name]
        ReportTest(name + ' (energy)', energy, data['energy'], 1e-9)
        compare(name, 'forces', forces, data['forces'], ids)
        compare(name, 'positions', positions, data['positions'], ids)
        if dostress:
            for i in range(6):
                ReportTest(name + '(stress %s)' % (i,), 
                           stress[i], data['stress'][i], 1e-9)
            compare(name, 'stresses', stresses, data['stresses'], ids)
    else:
        # Store new results
        data = {}
        data['energy'] = energy
        data['forces'] = forces[::keep]
        data['positions'] = positions[::keep]
        if dostress:
            data['stress'] = stress
            data['stresses'] = stresses[::keep]
        assert name not in data
        newresult[name] = data
        
def compare(name, quantity, a, b, ids):
    for i in range(len(a)):
        ii = ids[i]
        if ii % keep == 0:
            x = a[i]
            y = b[ii // keep]
            for j in range(len(x)):
                ReportTest('%i %s (%s[%i,%i])' % (world.rank, name, quantity, ii, j),
                           x[j], y[j], 1e-10, silent=True)

ReportTest.ExitAfterError()

# EMT
atoms = myfcc('Cu', pbc=True)
atoms.set_calculator(EMT())
test(atoms, 'Cu EMT periodic')

atoms = myfcc('Cu', pbc=False)
atoms.set_calculator(EMT())
test(atoms, 'Cu EMT free')

atoms = myfcc('Cu', pbc=(1,1,0))
atoms.set_calculator(EMT())
test(atoms, 'Cu EMT mixed')

atoms = myalloy(('Au', 'Cu'), pbc=True)
atoms.set_calculator(EMT())
test(atoms, 'AuCu3 EMT periodic')

atoms = myalloy(('Au', 'Cu'), pbc=False)
atoms.set_calculator(EMT())
test(atoms, 'AuCu3 EMT free')

# Lennard-Jones
atoms = myfcc('Cu', pbc=True)
atoms.set_calculator(LennardJones([29], [0.15], [2.7], 3*2.7, False))
test(atoms, 'LennardJones periodic')

atoms = myfcc('Cu', pbc=False)
atoms.set_calculator(LennardJones([29], [0.15], [2.7], 3*2.7, False))
test(atoms, 'LennardJones free')

atoms = myfcc('Cu', pbc=(1,1,0))
atoms.set_calculator(LennardJones([29], [0.15], [2.7], 3*2.7, False))
test(atoms, 'LennardJones mixed')

# Morse
elements = np.array([atomic_numbers['Ru'], atomic_numbers['Ar']])
epsilon = np.array([[5.720, 0.092], [0.092, 0.008]])
alpha = np.array([[1.475, 2.719], [2.719, 1.472]])
rmin = np.array([[2.110, 2.563], [2.563, 4.185]])
atoms = myfcc('Ar', pbc=True)
atoms.set_calculator(Morse(elements, epsilon, alpha, rmin))
test(atoms, 'Morse periodic', dostress=False)

atoms = myfcc('Ar', pbc=False)
atoms.set_calculator(Morse(elements, epsilon, alpha, rmin))
test(atoms, 'Morse free', dostress=False)

atoms = myfcc('Ar', pbc=(0,1,0))
atoms.set_calculator(Morse(elements, epsilon, alpha, rmin))
test(atoms, 'Morse mixed', dostress=False)

if 0:
    # Gupta is broken with Python 3
    atoms = myalloy(('Y', 'Pt'), pbc=True)
    atoms.set_calculator(Gupta(PtY_Gupta))
    test(atoms, 'Gupta Pt3Y periodic')

    atoms = myalloy(('Y', 'Pt'), pbc=False)
    atoms.set_calculator(Gupta(PtY_Gupta))
    test(atoms, 'Gupta Pt3Y free')

    atoms = myalloy(('Y', 'Pt'), pbc=(0,0,1))
    atoms.set_calculator(Gupta(PtY_Gupta))
    test(atoms, 'Gupta Pt3Y mixed')

# EMT2013
atoms = myalloy(('Y', 'Pt'), pbc=True)
atoms.set_calculator(EMT2013(PtY_EMT2013))
test(atoms, 'EMT2013 Pt3Y periodic')

atoms = myalloy(('Y', 'Pt'), pbc=False)
atoms.set_calculator(EMT2013(PtY_EMT2013))
test(atoms, 'EMT2013 Pt3Y free')

atoms = myalloy(('Y', 'Pt'), pbc=(0,0,1))
atoms.set_calculator(EMT2013(PtY_EMT2013))
test(atoms, 'EMT2013 Pt3Y mixed')



if firsttime:
    print("Writing data to", resultname)
    out = open(resultname, 'wb')
    pickle.dump(newresult, out, -1)
    out.close()
    
ReportTest.Summary()



from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from ase.lattice.cubic import FaceCenteredCubic
from .helpers import create_atoms
import numpy as np


def make_nanocrystal(size, centres, rotations, min_dist=2.0, unit=None,
                     symbol=None, latticeconstant=None):
    """Create a nanocrystalline sample.

    Parameters:
      size:      Size of the system, in Angstrom (either a number or three numbers).
      centres:   Positions of grains in scaled coordinates.
      rotations: Rotation matrices for the grains.

      min_dist (optional):
                 If two atoms are closer than this distance, one of them is removed
                 (default: 2.0).

    In addition, either the 'unit' or the 'symbol' parameter must be specified.
      unit (optional):
                 A unit cell for building the crystals (Atoms object).
                 MUST be orthorhombic!
      symbol (optional):
                 If unit is None, then an FCC crystal of this element is used.
      latticeconstant (optional):
                 If symbol is specified, this overrides the default lattice constant.
    """

    assert type(size) == float or type(size) == list
    if type(size) == float:
        size = np.array([size, size, size])
    elif type(size) == list:
        size = np.array(size)
    assert size.shape == (3,)
    assert len(centres) == len(rotations)
    centres *= size
    if unit is None:
        unit = FaceCenteredCubic(symbol=symbol, size=(1,1,1), pbc=True,
                                 latticeconstant=latticeconstant)

    return create_atoms(centres, rotations, unit, size, min_dist)




from __future__ import print_function
import asap3
from asap3.mpi import world
from asap3.testtools import ReportTest
import numpy as np

if world.size == 1:
    raise RuntimeError("Cannot test MPI communication on a single processor.")

if world.rank == 0:
    asap3.print_version(1)

world.barrier()

outdata = np.array([world.rank * 2], int)
indata = np.zeros(1, int)

to = world.rank + 1
if to >= world.size:
    to = 0
fr = world.rank - 1
if fr < 0:
    fr += world.size

print("%d: to=%d from=%d" % (world.rank, to, fr))

rq = world.send(outdata, to, block=False)
world.receive(indata, fr)
world.wait(rq)

ReportTest("Data received by %s from %s" % (world.rank, fr), indata[0], 2*fr, 0)


outdata = world.rank * np.arange(10000000)
indata = np.zeros(10000000, int)

to = world.rank + 1
if to >= world.size:
    to = 0
fr = world.rank - 1
if fr < 0:
    fr += world.size

print("%d: to=%d from=%d" % (world.rank, to, fr))

rq = world.send(outdata, to, block=False)
world.receive(indata, fr)
#world.wait(rq)
print("Request completed:", rq.test())
rq.wait()

assert (indata == fr * np.arange(10000000)).all()

s = world.sum(2.5 * world.rank)
ReportTest("Float sum", s, 2.5 * 0.5 * world.size * (world.size - 1), 1e-9)

s = world.sum(world.rank)
ReportTest("Integer sum", s, world.size * (world.size - 1) / 2, 0)

s = world.max(1.0 * world.rank)
ReportTest("Float max", s, world.size - 1, 0)

s = world.max(world.rank)
ReportTest("Int max", s, world.size - 1, 0)

print("Checking reduces on arrays...")
d0 = np.arange(10) + world.rank
f0 = d0 + 0.0

d = d0.copy()
f = f0.copy()
world.max(d)
assert (d == np.arange(10) + world.size - 1).all()
world.max(f)
assert (f == np.arange(10) + world.size - 1.0).all()

d = d0.copy()
f = f0.copy()
world.min(d)
assert (d == np.arange(10)).all()
world.min(f)
assert (f == np.arange(10)).all()

print("  ... passed.")

print("Checking new communicator")
newcomm = world.new_communicator(np.arange(2))
if newcomm:
    print("I am", newcomm.rank)
    s = newcomm.sum(world.rank+1)
    ReportTest("Integer sum", s, 3, 0)

world.barrier()
ReportTest.Summary()



from __future__ import print_function
from numpy import *
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from ase import data
from asap3.testtools import *
import time

#random.seed([114173, 98658])

listcutoff = 4.98409   # The EMT value for Cu
element = "Cu"
latconst = data.reference_states[data.atomic_numbers[element]]['a']

def checklist(nb, a, cut):
    error = 0
    #nb2 = FullNeighborList(a, cut)
    a2 = Atoms(a)
    #nb2 = NeighborList(cut, a2, 0, full=True)
    nb2 = FullNeighborList(cut, a2, 0)
    print("    Checking Cartesian positions")
    err = abs((a.get_positions() - a2.get_positions()).ravel())
    idx = argmax(err)
    at, co = idx//3, idx%3
    print("      err =", err[idx], "at", idx, ":", at, co)
    ReportTest("      Worst Cartesian position (%d,%d)" % (at, co),
               a.get_positions()[at,co],
               a2.get_positions()[at,co], 1e-10)
    #print "    Checking internal positions"
    #err = abs((a.GetInternalPositions() - a2.GetInternalPositions()).flat)
    #idx = argmax(err)
    #at, co = idx/3, idx%3
    #print "      err =", err[idx], "at", idx, ":", at, co
    #ReportTest("      Worst internal position (%d,%d)" % (at, co),
    #           a.GetInternalPositions()[at,co],
    #           a2.GetInternalPositions()[at,co], 1e-10)
    print("    Checking full list")
    for i in range(len(a)):
        l1 = list(nb[i])
        l1.sort()
        l2 = list(nb2[i])
        l2.sort()
        if not l1 == l2:
            ReportTest.BoolTest("NB lists for atom %d should be identical"
                                % (i,), False)
            print("nb1[%d] = %s" % (i, str(l1)))
            print("nb2[%d] = %s" % (i, str(l2)))
            print("pos1[%d] = %s" % (i, str(a.get_positions()[i])))
            print("pos2[%d] = %s" % (i, str(a2.get_positions()[i])))
            error += 1
            if error > 3:
                print("Too many errors, giving up!")
                raise RuntimeError("Too many errors, giving up!")

for i in range(10):
    print()
print_version(1)
#print "RandomArray seeds:", RandomArray.get_seed()
pbc = tuple(random.random_integers(0,1,3))
print("Periodic boundaries:", pbc)

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(10,10,10),
                          symbol=element, pbc=pbc, debug=0)

nblist = NeighborList(listcutoff, atoms, 0, full=True)

# This would only work with full PBC
#ReportTest("Total neighborlist length", totallength, 42*len(atoms), 0)

if 1:
    print()
    print("Checking number of affected atoms.")
    #Verbose(1)

    np_atoms = Atoms(atoms, pbc=False)
    np_nblist = NeighborList(listcutoff, np_atoms, 0, full=True)

    i = random.random_integers(0, len(np_atoms)-1)
    dr = random.standard_normal(3)
    dr *= 10.0/sqrt(dot(dr,dr))
    atom = np_atoms[i]
    oldpos = array(atom.position)
    newpos = oldpos + dr
    atom.position = newpos
    nr = np_nblist.test_partial_update(array([i,], int32), np_atoms)
    # Count who should be affected.
    r = np_atoms.get_positions() - oldpos
    r2 = sum(r * r, 1)
    a = less_equal(r2, listcutoff*listcutoff)
    r = np_atoms.get_positions() - newpos
    r2 = sum(r * r, 1)
    b = less_equal(r2, listcutoff*listcutoff)

    nr2 = sum(logical_or(a,b))
    print(nr, "atoms were affected")
    print(sum(a), "near old position and", sum(b), "near new position.")
    print(nr2, "atoms should be affected")
    ReportTest("Number of affected atoms", nr, nr2, 0)
    
    del np_atoms, np_nblist
    
if 1:
    print()
    print("Testing perturbations of all the atoms")

    magnarr = array((0.0, 0.01, 0.1, 1.0, 3.0, 10.0))
    pick = argsort(random.random((len(magnarr),)))
    for magn in take(magnarr, pick):
        dx = magn * random.uniform(-1, 1, (len(atoms), 3))
        print("  Perturbation:", magn)
        atoms.set_positions(dx + atoms.get_positions())
        nblist.check_and_update(atoms)
        checklist(nblist, atoms, listcutoff)



print() 
print("Testing perturbations of single atoms")

magnarr = array((0.0, 0.01, 0.1, 1.0, 3.0, 10.0))
pick1 = argsort(random.random((len(magnarr),)))
for magn in take(magnarr, pick1):
    numarr = array((1, 3, 10, len(atoms)//2, -3))
    pick2 = argsort(random.random((len(numarr),)))
    for number in take(numarr, pick2):
        # Pick number random atoms.
        if number < 0:
            # Find N neighboring atoms and perturb them
            number = -number
            n = 0
            while len(nblist[n]) < number:
                n += 1
            pick = concatenate([array((n,)), nblist[n][:number-1]])
        else:
            pick = argsort(random.random((len(atoms),)))[:number]
        if number > 15:
            s = "<<< %s atoms >>>" % (number,)
        else:
            s = str(list(pick))
        print("  dr = %.3f: %d atoms: %s" % (magn, number, s))
        for i in pick:
            atom = atoms[i]
            dr = random.standard_normal(3)
            dr *= magn/sqrt(dot(dr,dr))
            atom.position += dr

        nblist.test_partial_update(pick.astype(int32), atoms)
        checklist(nblist, atoms, listcutoff)


ReportTest.Summary()




from __future__ import print_function
from __future__ import absolute_import
import asap3
import ase.parallel
import ase.constraints as _ase_constraints
import asap3.constraints as _asap_constraints
#from ase.io.trajectory import PickleTrajectory as _PickleTrajectory
from ase.io import trajectory
from . import cpu_setup
import numpy as np
import ase.parallel


class _GetAtoms:
    "Mixin class implementing the get_atoms method."
    def get_atoms(self, n, cpus=None, extra=None):
        """Return Atoms object number n from the trajectory.

        traj.get_atoms(n) is very similar to traj[n], but with the following
        differences:

        In serial simulations are traj.get_atoms(n) and traj[n] equivalent.

        In parallel simulations:
        traj[n] returns all atoms on all processors

        traj.get_atoms(n) returns all atoms on the master, and None on
        all other processors.

        traj.get_atoms(n, cpus) returns a ParallelAtoms object with
        atoms distributed among the processors, and processor layout
        given by cpus (three integers).
        
        Optional parameters:
        extra (BundleTrajectory only): 
            A list of names of extra arrays that should be read from the 
            input file.  The data is available with atoms.get_array.
        """
        if self.master:
            atoms = self[n]
            if extra:
                for d in extra:
                    atoms.set_array(d, self.read_extra_data(d, n))
        else:
            atoms = None

        if cpus is None:
            return atoms
        elif cpus == 'auto':

            if self.master:
                num_cores = ase.parallel.world.size
                print("num cpus:", num_cores)
                print("cell:", np.diag(atoms.cell).tolist())
                cpus = cpu_setup.calculate_optimal_cpu_setup(num_cores, np.diag(atoms.cell).tolist())
                print("cpu setup:", cpus)

            cpus = ase.parallel.broadcast(cpus, 0)
            return asap3.MakeParallelAtoms(atoms, cpus)
        else:
            return asap3.MakeParallelAtoms(atoms, cpus)


class PickleTrajectory(trajectory.PickleTrajectory, _GetAtoms):
    write_forces = False
    write_charges = False
    write_magmoms = False

    def __init__(self, filename, mode='r', atoms=None, master=None,
                 *args, **kwargs):
        if master is not None:
            if atoms.get_comm().rank != 0:
                raise NotImplementedError("It is required that the cpu with rank 0 is the master")
        trajectory.PickleTrajectory.__init__(self, filename, mode, atoms, master, *args, **kwargs)

    def set_atoms(self, atoms=None):
        if atoms is not None and getattr(atoms, "parallel", False):
            atoms = asap3.Collector(atoms, self.master)
            self.sanitycheck = False
        trajectory.PickleTrajectory.set_atoms(self, atoms)
        
    def write(self, atoms=None):
        if atoms is not None and getattr(atoms, "parallel", False):
            atoms = asap3.Collector(atoms, self.master)
            self.sanitycheck = False
        trajectory.PickleTrajectory.write(self, atoms)



def Trajectory(filename, mode='r', atoms=None, master=None, 
               properties=['energy']):
    """A Trajectory can be created in read, write or append mode.

    Parameters:

    filename:
        The name of the parameter file.  Should end in .traj.

    mode='r':
        The mode.

        'r' is read mode, the file should already exist, and
        no atoms argument should be specified.

        'w' is write mode.  If the file already exists, it is
        renamed by appending .bak to the file name.  The atoms
        argument specifies the Atoms object to be written to the
        file, if not given it must instead be given as an argument
        to the write() method.

        'a' is append mode.  It acts a write mode, except that
        data is appended to a preexisting file.

    atoms=None:
        The Atoms object to be written in write or append mode.

    master=None:
        Controls which process does the actual writing. The
        default is that process number 0 does this.  If this
        argument is given, processes where it is True will write.

    properties=['energy']:
        List of properties that should be stored in the 
        Trajectory file.  Unlike the ASE version, the default 
        is to only store the energy, not forces, stress etc.  
        Allowed values are energy, forces, stress, dipole, charges,
        magmom and magmoms; but only the first three are meaningful
        in an Asap calcualtion. 

    The atoms, master and properties arguments are ignores in read
    mode.
    """
    if mode == 'r':
        return TrajectoryReader(filename)
    return TrajectoryWriter(filename, mode, atoms, master=master, properties=properties)


class TrajectoryReader(trajectory.TrajectoryReader, _GetAtoms):
    def __init__(self, filename):
        trajectory.TrajectoryReader.__init__(self, filename)
        self.master = ase.parallel.world.rank == 0

    def close(self):
        # Do nothing - fixes bug 99 in ASE
        pass
    
    def __getitem__(self, i=-1):
        atoms = trajectory.TrajectoryReader.__getitem__(self, i)
        constraints = atoms.constraints
        for i, c in enumerate(constraints):
            print("XXXX", c.__class__)
            if c.__class__ is _ase_constraints.FixAtoms:
                # Exact class match, not a subclass
                constraints[i] = _asap_constraints.FixAtoms(indices=c.index)
        return atoms

class TrajectoryWriter(trajectory.TrajectoryWriter):
    def __init__(self, filename, mode='w', atoms=None, properties=None,
                 extra=[], master=None):
        self.allow_forces = properties is not None and 'forces' in properties
        if master is not None:
            if atoms.get_comm().rank != 0:
                raise NotImplementedError("It is required that the cpu with rank 0 is the master")
        else:
            if atoms and hasattr(atoms, 'get_comm'):
                rank = atoms.get_comm().rank
            else:
                rank = ase.parallel.rank
            master = (rank == 0)
        if atoms is not None and getattr(atoms, "parallel", False):
            atoms = asap3.Collector(atoms, master, self.allow_forces)        
        trajectory.TrajectoryWriter.__init__(self, filename, mode=mode, atoms=atoms, 
                                       properties=properties, extra=extra, master=master)
       
    def write(self, atoms=None, **kwargs):
        if atoms is not None and getattr(atoms, "parallel", False):
            atoms = asap3.Collector(atoms, self.master, self.allow_forces)
        elif hasattr(self.atoms, "_is_asap_collector_object"):
            self.atoms.reset_collector()
        trajectory.TrajectoryWriter.write(self, atoms, **kwargs)




from __future__ import print_function
# Testing various potentials.

from asap3 import *
from asap3.md.verlet import VelocityVerlet
from asap3.EMT2013Parameters import sihb_PtY_parameters
from ase.lattice.cubic import *
from ase.lattice.compounds import *
try:
    from ase.data import atomic_masses_legacy
except ImportError:
    atomic_masses_legacy = None
from numpy import *
from asap3.testtools import ReportTest
try:
    import potResults
except ImportError:
    resultfail = True
else:
    resultfail = False
import os

timeunit = 1.018047e-14             # Seconds
femtosecond = 1e-15 / timeunit      # Femtosecond in atomic units

print_version(1)


def dotest(atoms, nsteps, ampl, name):
    if atomic_masses_legacy is not None:
        atoms.set_masses(atomic_masses_legacy[atoms.numbers])
    print("Potential energy", atoms.get_potential_energy() / len(atoms))
    r = atoms.get_positions()
    r.flat[:] += ampl * sin(arange(3*len(atoms)))
    atoms.set_positions(r)
    print("Potential energy", atoms.get_potential_energy() / len(atoms))

    print("Running Verlet dynamics (%s)" % (name,))
    dyn = VelocityVerlet(atoms, 2*femtosecond)
    etot1 = (atoms.get_potential_energy() + atoms.get_kinetic_energy())
    dyn.run(nsteps)
    etot2 = (atoms.get_potential_energy() + atoms.get_kinetic_energy())
    ReportTest(("Energy conservation (%s)" % (name,)), etot1, etot2, 1.0)
    print(etot1, etot2)

    epot = atoms.get_potential_energies()
    stress = atoms.get_stresses()
    if firsttime:
        print("Reporting energies and stresses")
        e = []
        s = []
        j = 0
        for i in range(0, len(atoms), 100):
            e.append(epot[i])
            s.append(stress[i,j])
            j = (j + 1) % 6
        print("e"+name+" =", repr(e), file=out)
        print("s"+name+" =", repr(s), file=out)
    else:
        print("Testing energies and stresses")
        j = 0
        eres=getattr(potResults, "e"+name)
        sres=getattr(potResults, "s"+name)
        for i in range(len(atoms)//100):
            ReportTest(("%s energy %d" % (name, i*100)),
                       epot[i*100], eres[i], 1e-8, silent=True)
            ReportTest(("%s stress %d" % (name, i*100)),
                       stress[i*100, j], sres[i], 1e-8, silent=True)
            j = (j + 1) % 6

firsttime = (len(sys.argv) >= 2 and sys.argv[1] == '--first')
if firsttime and os.path.exists("potResults.py"):
    print("This will overwrite the result file 'potResults.py'.")
    print("If you really want to do this, erase it and run this again.")
    sys.exit(1)
if resultfail and not firsttime: 
    print("Importing 'potResults.py' failed!")
    print("Maybe you need to create it with the --first option.")
    sys.exit(1)
    
if firsttime:
    print("Creating the file 'potResults.py'")
    out = open('potResults.py', "w")    

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(15,15,15),
                          symbol="Cu", pbc=(1,0,1), debug=0)
ReportTest("Number of Cu atoms", len(atoms), 13500, 0)

atoms.set_calculator(EMT())
dotest(atoms, 50, 0.1, "EMT_Cu")

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(15,15,15),
                          symbol="Cu", pbc=(1,0,1), debug=0)
ReportTest("Number of Cu atoms", len(atoms), 13500, 0)

atoms.set_calculator(EMT(EMTRasmussenParameters()))
dotest(atoms, 50, 0.1, "EMT_Cu_Rasm")

#atoms = BodyCenteredCubic([[1,0,0],[0,1,0],[0,0,1]], size=(15,15,30),
#                          element="Mo", periodic=(1,0,1), debug=0)
#ReportTest("Number of Mo atoms", len(atoms), 13500, 0)
#
#atoms.SetCalculator(MoPotential())
#dotest(atoms, 50, 0.06, "Mo")

atoms = L1_2(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(15,15,15),
             symbol=("Cu", "Au"), latticeconstant=3.95, pbc=(1,0,1), 
             debug=0)
ReportTest("Number of alloy atoms", len(atoms), 13500, 0)
nCu = sum(equal(atoms.get_atomic_numbers(), 29))
nAu = sum(equal(atoms.get_atomic_numbers(), 79))
ReportTest("Number of Cu atoms in alloy", nCu, 13500/4, 0)
ReportTest("Number of Au atoms in alloy", nAu, 3*13500/4, 0)

atoms.set_calculator(EMT())
dotest(atoms, 50, 0.06, "EMT_CuAu3")

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(15,15,15),
                          symbol="Pt", pbc=(1,0,1), debug=0)
ReportTest("Number of Pt atoms", len(atoms), 13500, 0)

atoms.set_calculator(EMT2013(sihb_PtY_parameters))
dotest(atoms, 50, 0.1, "EMT2013_Pt")

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(15,15,15),
                          symbol="Y", latticeconstant=4.97,
                          pbc=(1,0,1), debug=0)
ReportTest("Number of Y atoms", len(atoms), 13500, 0)

atoms.set_calculator(EMT2013(sihb_PtY_parameters))
dotest(atoms, 50, 0.1, "EMT2013_Y")

atoms = L1_2(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(15,15,15),
             symbol=("Y", "Pt"), latticeconstant=4.06, pbc=(1,0,1), 
             debug=0)
ReportTest("Number of alloy atoms", len(atoms), 13500, 0)
nY = sum(equal(atoms.get_atomic_numbers(), 39))
nPt = sum(equal(atoms.get_atomic_numbers(), 78))
ReportTest("Number of Cu atoms in alloy", nY, 13500/4, 0)
ReportTest("Number of Au atoms in alloy", nPt, 3*13500/4, 0)

atoms.set_calculator(EMT2013(sihb_PtY_parameters))
dotest(atoms, 50, 0.06, "EMT2013_Pt3Y")

if firsttime:
    # Create the "main" routine in the results module.
    print('if __name__ == "__main__":', file=out)
    print('    print "This is not a test, but a module containing test results"', file=out)
    out.close()
    
ReportTest.Summary()




from __future__ import print_function
from numpy import *
from asap3 import *
from ase.lattice.cubic import SimpleCubic
from asap3.testtools import *
import time
import commands
import re

optimize = True
element = "Cu"
systemsizes = [(5,5,5), (10,10,10), (15, 15, 15), (20, 20, 25), (46,46,47), (100,100,100)]
#systemsizes = [(5,5,5), (10,10,10)]
nsteps = (1000, 1000, 1000, 1000, 1000, 100)
kT = 0.1
logfilename = "montecarlotiming.log"

def MakeSystem(size, element, perturbation):
    lc = data.reference_states[data.atomic_numbers[element]]['a'] / 2.0**(1.0/3.0)
    
    atoms = MonteCarloAtoms(SimpleCubic(directions=((1,0,0),(0,1,0),(0,0,1)),
                                        size=size, symbol=element,
                                        latticeconstant=lc,
                                        pbc=True))
    rs = atoms.get_scaled_positions()
    atoms.set_scaled_positions(0.5 * (rs + rs*rs))
    r = atoms.get_positions()
    dr = perturbation * random.standard_normal(r.shape)
    atoms.set_positions(r + dr)
    return atoms


percentages = []
sizes = []
timings = []

print_version()

for i,size in enumerate(systemsizes):
    atoms = MakeSystem(size, element, 0.1)
    n = len(atoms)
    #p1 = RasMol(atoms.Repeat((2,2,2)))
    # Pregenerate the trial steps as not to time random
    trial_id = random.randint(0, len(atoms), (nsteps[i],))
    lc = data.reference_states[data.atomic_numbers[element]]['a'] / 2.0**(1.0/3.0)
    trial_move = random.normal(0, lc, (nsteps[i],3))
    if optimize:
        atoms.set_calculator(MonteCarloEMT())
    else:
        atoms.set_calculator(EMT())
    eorig = e = atoms.get_potential_energy()

    nrej = 0
    startcpu, startwall = time.clock(), time.time()
    for j in range(nsteps[i]):
        atom = atoms[trial_id[j]]
        oldpos = atom.get_position()
        atom.set_position(oldpos + trial_move[j])
        newe = atoms.get_potential_energy()
        de = newe - e
        #print de
        if de > 500.0*kT:
            de = 500.0*kT
        if (de > 0.0 and random.random() > exp(-de/kT)):
            atom.set_position(oldpos)
            nrej = nrej + 1
        else:
            e = newe
    tcpu, twall = time.clock() - startcpu, time.time() - startwall
    atoms.set_scaled_positions(atoms.get_scaled_positions())
    #p2 = RasMol(atoms.Repeat((2,2,2)))
    pc = 100.0 * tcpu / twall
    timing = 1e6 * tcpu / nsteps[i]

    print()
    print("Acceptance ratio:", 100.0 * (nsteps[i]-nrej) / nsteps[i])
    print("Energy change:", atoms.get_potential_energy() - eorig, "eV")
    print("CPU usage:", pc, "%")
    print("Number of atoms:", n)
    print("Timing:", timing, "usec/step")
    percentages.append(pc)
    timings.append(timing)
    sizes.append(n)
    
now=time.strftime("%Y/%m/%d %H:%M")
asapversion = get_version()
version = asapversion.split()[2]
compiler = asapversion.split("'")[1]
host = commands.getoutput("hostname")
if re.match("^n\d\d\d.dcsc.fysik.dtu.dk$", host):
    print("    This is a d512 node on Niflheim.")
    fullhost = "niflheim-d512/%s" % (host.split(".")[0])
    host = "niflheim-d512"
elif re.match("^[stu]\d\d\d.dcsc.fysik.dtu.dk$", host):
    print("    This is an s50 node on Niflheim.")
    fullhost = "niflheim-s50/%s" % (host.split(".")[0])
    host = "niflheim-s50"
else:
    fullhost = host


loginfo = """Running on %s at %s
  Asap version: %s
  Monte Carlo Optimization: %s
""" % (fullhost, now, version, str(optimize))
for i in range(len(percentages)):
    loginfo = loginfo + ("    %d atoms: %.1f usec/step  (CPU usage: %.1f)\n"
                         % (sizes[i], timings[i], percentages[i]))

print("\nLog info:\n")
print(loginfo)
logfile = open(logfilename, "a")
logfile.write(loginfo+"\n")
logfile.close()



from __future__ import print_function
from asap3 import *
from numpy import *
from asap3.testtools import ReportTest
from ase.lattice.cubic import *
from OpenKIM_modelname import openkimmodel

element = 'Au'
sizes = arange(10,0,-1)

energy = None
force = None


for s in sizes:
    atoms = FaceCenteredCubic(symbol=element, size=(s,s,s), pbc=True)
    uc = atoms.get_cell()
    #uc[0], uc[1] = uc[1].copy(), uc[0].copy()
    #atoms.set_cell(uc, scale_atoms=False)
    uc[2] *= -1.0
    atoms.set_cell(uc, scale_atoms=True)
    r = atoms.get_positions()
    for i in range(0, len(r), 4):
        r[i][0] += 0.1
    atoms.set_positions(r)
    atoms.set_calculator(EMT())
    e = atoms.get_potential_energy() / len(atoms)   
    print("%5i atoms: E = %.5f eV/atom" % (len(atoms), e))
    if energy is None:
        energy = e
    else:
        ReportTest("Energy for size %i (%i atoms)" % (s, len(atoms)),
                   e, energy, 1e-8)
    f = atoms.get_forces()
    if force is None:
        force = f[:4]
    else:
        for i in range(len(f)):
            for j in range(3):
                ReportTest("Force for size %i atom %i of %i component %i"
                           % (s, i, len(atoms), j),
                           f[i,j], force[i % 4, j], 1e-8, silent=True)
        #print force
        #print f[:10]
    del atoms

ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from ase.optimize import LBFGS
import numpy as np

calc = EMT()
symbol = 'Cu'

syms = ((True, True, True),
        (False, False, False),
        (True, True, False),
        (True, False, False))

for symmetry in syms:
    print("Periodicity:", symmetry)
    for n in (4,6):
        atoms = FaceCenteredCubic(symbol=symbol, size=(4, 8, n), pbc=symmetry)
        atoms.set_calculator(calc)

        dyn = LBFGS(atoms, trajectory=None, logfile=None)
        dyn.run(fmax=0.05)

        e = atoms.get_potential_energy()
        print("Layers, energy: %2i %8.3f" % (n, e))

print("TEST PASSED.")



from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                          size=(6,6,6), symbol="Cu")
atoms.set_calculator(EMT())
f1 = atoms.get_forces()
atoms.set_calculator(EMT())
f2 = atoms.get_forces()
maxdev = abs(f2 - f1).max()
print(maxdev)
ReportTest("Max error 1:", maxdev, 0.0, 1e-6)

atoms2 = Atoms(atoms)
if atoms2.get_calculator() is None:
    # Slightly old ase
    atoms2.set_calculator(atoms.get_calculator())
f2 = atoms2.get_forces()
maxdev = abs(f2 - f1).max()
print(maxdev)
ReportTest("Max error 2:", maxdev, 0.0, 1e-6)

f2 = atoms.get_forces()
maxdev = abs(f2 - f1).max()
print(maxdev)
ReportTest("Max error 1:", maxdev, 0.0, 1e-6)




from __future__ import print_function
from __future__ import division
from asap3 import *
import pickle
from numpy import *
from ase.build import bulk
from asap3.testtools import ReportTest
import ase.data
from ase.eos import EquationOfState
import numpy as np
from asap3.mpi import world

print_version(1)

verbose = False

model = 'ex_model_Ar_SLJ_MultiCutoff'   # Requires multiple neighbor lists.
sizes = [(25,25,25)]

if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]
    
if OpenKIMsupported:
    try:
        calc = OpenKIMcalculator(model)
    except AsapError as oops:
        if oops.args[0].startswith('Failed to initialize OpenKIM model'):
            print("OpenKIM model {} not installed - skipping test.".format(model))
            calc = None
        else:
            raise

if OpenKIMsupported and calc is not None:
    ismaster = world.rank == 0
    for size in sizes:
        if ismaster:
            atoms = bulk('Ar', 'fcc', 5.26).repeat(size)
        else:
            atoms = None
        atoms = MakeParallelAtoms(atoms, cpulayout)
        calc = OpenKIMcalculator(model)   # Reuse of OpenKIM objects not yet supported.
        atoms.set_calculator(calc)

        e = atoms.get_potential_energy()/atoms.get_number_of_atoms()
        if ismaster:
            print("Potential energy:", e)
        ReportTest("Potential energy {}".format(str(size)), e, -0.0104, 1e-6)

        eq_cell = atoms.get_cell()
        scales = np.linspace(0.99, 1.01, 7)
        energies = []
        volumes = []
        for s in scales:
            atoms.set_cell(s * eq_cell, scale_atoms=True)
            energies.append(atoms.get_potential_energy())
            volumes.append(atoms.get_volume())
        eos = EquationOfState(volumes, energies)
        v0, e0, B = eos.fit()
        v_cell = v0 * 4 / atoms.get_number_of_atoms()
        a0 = v_cell**(1/3)
        if ismaster:
            print("Lattice constant:", a0)
            print("Bulk modulus:", B)
        ReportTest("Lattice constant {}".format(str(size)), a0, 5.26, 1e-5)
        



from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest
import numpy as np
from numpy.random import RandomState
from ase.data import reference_states, atomic_numbers, chemical_symbols
from ase.build import bulk
import sys

brokenmodels = [
    'ex_model_Ar_P_Morse_MultiCutoff',                           # Does not set Destroy pointer.
    'EAM_IMD_BrommerGaehler_2006B_AlNiCo__MO_128037485276_003',  # Force and energy inconsistent
    'EAM_NN_Johnson_1988_Cu__MO_887933271505_002',               # Force and energy inconsistent
    'EAM_Magnetic2GQuintic',                                     # Crash with Intel compiler.
    'EAM_MagneticCubic',                                         # Crash with Intel compiler.
    'amp_parametrized_model',                                    # Experimental
    ]

# Some models require a specific primary and secondary element,
# e.g. if it is aiming at impurity calculations, and using the wrong
# element as host brings it outside its comfort zone.
explicit_elements = {
    'EAM_Dynamo_HepburnAckland_2008_FeC__MO_143977152728_005': ('Fe', 'C'),   # C impurity in Fe
    'EAM_Dynamo_SmirnovaKuskinStarikov_2013_UMoXe__MO_679329885632_005': ('U', 'Mo'),
    }
    
tolerance = 1e-4
#set_verbose(1)

if OpenKIMsupported:
    openkimmodels = OpenKIMavailable()
else:
    openkimmodels = []
    print("OpenKIM support is not compiled into Asap.")
    
#rnd = RandomState(42)  # We want deterministic random numbers
rnd = RandomState()

known_states = ['bcc', 'fcc', 'hcp', 'diamond', 'sc']

delta = 0.001

if len(sys.argv) > 1:
    openkimmodels = sys.argv[1:]
    brokenmodels = []

skipped = []
crashed = []

#rnd.shuffle(openkimmodels)

for model in openkimmodels:
    broken = False
    for brk in brokenmodels:
        if model.startswith(brk):
            broken = True
    if broken:
        print("\nSkipping broken KIM model:", model)
        skipped.append((model, "Blacklisted"))
        continue
    print("\nKIM model:", model)
    if model == 'EMT':
        calculator = EMT()
        elements = ('Cu', 'Au', 'Ag')
    else:
        calculator = OpenKIMcalculator(model)
        elements = calculator.get_supported_elements(user=False)
    print("Supported elements:", elements)
    overrule_elements = model in explicit_elements
    if overrule_elements:
        elements = explicit_elements[model]
    if len(elements) == 0:
        print("No standard elements supported - SKIPPING MODEL!")
        continue
    elif len(elements) == 1:
        main = elements[0]
        other = None
        state = reference_states[atomic_numbers[main]]
    else:
        if not overrule_elements:
            elements = list(elements)
            rnd.shuffle(elements)
        for i in range(len(elements)):
            main = elements[i]
            other = elements[i-1]
            state = reference_states[atomic_numbers[main]]
            if state and state['symmetry'] in known_states:
                break
    if state['symmetry'] not in known_states:
        print("Cannot simulate {}, reference state '{}' not supported".format(main, state['symmetry']))
        print("SKIPPING MODEL!")
        continue

    for case in range(2):
        if case == 0:
            init_atoms = bulk(main, orthorhombic=True).repeat((7,7,7))
        elif case == 1:
            init_atoms = bulk(main).repeat((1,2,7))
            # We cannot reuse the OpenKIMcalculator
            if model != "EMT":
                calculator = OpenKIMcalculator(model)
        else:
            raise RuntimeError("Unknown case "+str(case))
        r = init_atoms.get_positions()
        r += rnd.normal(0.0, 0.1, r.shape)
        init_atoms.set_positions(r)
        z = init_atoms.get_atomic_numbers()
        if other:
            some_atoms = rnd.randint(0, 20, len(init_atoms)) == 0
            z[some_atoms] = atomic_numbers[other]
            init_atoms.set_atomic_numbers(z)
            z_other = atomic_numbers[other]
        else:
            z_other = 0
        print ("Generated a %s system with %i %s-atoms and %i %s-atoms"
                   % (state['symmetry'], 
                          np.equal(z, atomic_numbers[main]).sum(),
                          main,
                          np.equal(z, z_other).sum(),
                          other))
        print("Lattice constant a =", state['a']) 
        old_energy = old_forces = None
        rndat = rnd.randint(len(init_atoms))



        #print "Testing %s with %s" % (model, nbl)
        atoms = Atoms(init_atoms)
        try:
            atoms.set_calculator(calculator)
            e = atoms.get_potential_energy()
            f = atoms.get_forces()
        except (RuntimeError, ValueError, AsapError) as exc:
            txt = "{} (case {}) raised exception {}: {}".format(model, case, str(type(exc)), str(exc))
            print(txt)
            crashed.append(txt)
            continue

        atoms[rndat].position[0] += delta
        de = atoms.get_potential_energy() - e
        f = 0.5 * f + 0.5 * atoms.get_forces()
        exp_de = -delta * f[rndat,0]
        #print "Old energy: %.9f.   Change: %.9f    Expected: %.9f   Abs: %.9e   Relative: %.9f" % (e, de, exp_de, de-exp_de, (de-exp_de)/exp_de)
        ReportTest("{} (case {}) force consistent".format(model, case), de, exp_de, tolerance)

    
print("Skipped models:")
if skipped:
    for m in skipped:
        print("   {}: \t{}".format(*m))
else:
    print("    None")
print("Crashing models:")
if crashed:
    for m in crashed:
        print("   ", m)
else:
    print("    None")
ReportTest.Summary()



from __future__ import print_function
from numpy import *
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from ase import data
from asap3.testtools import *
import time
import sys

if 0:
    seed = [345763, 166945 + 1000]
    for i in range(10):
        print()
    print("Random seed:", seed)
    random.seed(seed)
    ReportTest.ExitAfterError(True)
        
longtest = (len(sys.argv) >= 2 and sys.argv[1] == '--long')
if longtest:
    print("Running extra long test.")
    ReportTest.ExitAfterError(True)
    del sys.argv[1]
    
listcutoff = 4.98409   # The EMT value for Cu
element = "Cu"
latconst = data.reference_states[data.atomic_numbers[element]]['a']

def checkenergies(a, a2):
    err = 0
    if 0:
        print("    Checking Cartesian positions")
        err = abs((a.get_positions() - a2.get_positions()).ravel())
        idx = argmax(err)
        at, co = idx/3, idx%3
        print("      err =", err[idx], "at", idx, ":", at, co)
        ReportTest("      Worst Cartesian position (%d,%d)" % (at, co),
                   a.get_positions()[at,co],
                   a2.get_positions()[at,co], 1e-10)
    print("    Checking energies")
    e1 = a.get_potential_energies()
    e2 = a2.get_potential_energies()
    err = e1 - e2
    idx = argmax(err)
    ReportTest("      Worst energy (%s)" % (idx,), e1[idx], e2[idx], 1e-5)
    
print_version(1)
#print "RandomArray seeds:", RandomArray.get_seed()
if len(sys.argv) >= 2 and sys.argv[1] == '--pbc':
    if len(sys.argv) < 3:
        raise ValueError("No argument to --pbc")
    pbcch = sys.argv[2]
    if len(pbcch) != 3:
        raise ValueError("Argument to pbc should be a string of three digits.")
    pbc = []
    for i in (0,1,2):
        pbc.append(int(pbcch[i]))
    print("Periodic boundary conditions forced!")
    pbcrnd = False
else:
    pbc = None
    pbcrnd = True
    
#Verbose(1)

if longtest:
    magnarr = array([0.0, 0.01, 0.1] + range(1, 11))
    numarr = array(range(1, 15) + range(50, 2000, 100) + [-3,])
    nrun = 10
else:
    magnarr = array((0.0, 0.01, 0.1, 1.0, 3.0, 10.0))
    numarr = array((1, 3, 10, 50, 200, 2000, -3))
    nrun = 1

for ntest in range(nrun):
    print()
    print("Running pass", ntest+1, "of", nrun)
    print()
    if pbcrnd:
        pbc =   tuple(random.randint(0,2,3))
    print("Periodic boundaries:", pbc)
    if nrun > 1:
        time.sleep(1)
    atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(10,10,10),
                              symbol=element, pbc=pbc)
    atoms = MonteCarloAtoms(atoms)
    print("Number of atoms:", len(atoms))
    atoms.set_calculator(MonteCarloEMT())
    atoms.get_potential_energies()  # Force creation of neighbor list.
    atoms2 = atoms.copy()
    atoms2.set_calculator(EMT())

    print() 
    print("Testing perturbations of single atoms")

    pick1 = argsort(random.random((len(magnarr),)))
    for magn in take(magnarr, pick1):
        pick2 = argsort(random.random((len(numarr),)))
        for number in take(numarr, pick2):
            # Pick number random atoms.
            if number < 0:
                # Find N neighboring atoms and perturb them
                number = -number
                nblist = atoms.get_calculator().get_neighborlist()
                n = 0
                while len(nblist[n]) < number:
                    n += 1
                pick = concatenate([array((n,)), nblist[n][:number-1]])
                del nblist
            else:
                pick = argsort(random.random((len(atoms),)))[:number]
            if number > 15:
                s = "<<< %s atoms >>>" % (number,)
            else:
                s = str(list(pick))
            print("  dr = %.3f: %d atoms: %s" % (magn, number, s))
            for i in pick:
                dr = random.standard_normal(3)
                dr *= magn/sqrt(dot(dr,dr))
                atom = atoms[i]
                atom.position = atom.position + dr
                atom = atoms2[i]
                atom.position = atom.position + dr
            checkenergies(atoms, atoms2)


ReportTest.Summary()




from __future__ import print_function
from __future__ import division
from asap3 import *
import pickle
from numpy import *
from ase.build import bulk
from asap3.testtools import ReportTest
import ase.data
from ase.eos import EquationOfState
import numpy as np

print_version(1)

verbose = False

model = 'ex_model_Ar_P_Morse'
sizes = [(1,1,1), (2,2,2), (5,5,5)]

if OpenKIMsupported:
    try:
        calc = OpenKIMcalculator(model)
    except AsapError as oops:
        if oops.args[0].startswith('Failed to initialize OpenKIM model'):
            print("OpenKIM model {} not installed - skipping test.".format(model))
            calc = None
        else:
            raise

if OpenKIMsupported and calc is not None:
    for size in sizes:
        atoms = bulk('Ar', 'fcc', 5.26).repeat(size)
        calc = OpenKIMcalculator(model)   # Reuse of OpenKIM objects not yet supported.
        atoms.set_calculator(calc)

        e = atoms.get_potential_energy()/len(atoms)
        print("Potential energy:", e)
        ReportTest("Potential energy {}".format(str(size)), e, -0.092798, 1e-5)

        eq_cell = atoms.get_cell()
        scales = np.linspace(0.97, 1.03, 7)
        energies = []
        volumes = []
        for s in scales:
            atoms.set_cell(s * eq_cell, scale_atoms=True)
            energies.append(atoms.get_potential_energy())
            volumes.append(atoms.get_volume())
        eos = EquationOfState(volumes, energies)
        v0, e0, B = eos.fit()
        v_cell = v0 * 4 / len(atoms)
        a0 = v_cell**(1/3)
        print("Lattice constant:", a0)
        print("Bulk modulus:", B)
        ReportTest("Lattice constant {}".format(str(size)), a0, 5.2539, 1e-4)
        



"""Test for memory leaks in the CNA code."""
from __future__ import print_function

import asap3
import asap3.analysis
from ase.lattice.compounds import L1_2
import numpy as np

steps = 30
msteps = 5

def makeatoms():
    atoms = L1_2(size=(10,10,10), symbol=('Au', 'Cu'), latticeconstant=4.0)
    r = atoms.get_positions()
    r += np.random.normal(0.0, 0.0001, r.shape)
    atoms.set_positions(r)
    return atoms

m0 = asap3.heap_mallinfo()
if m0 < 0:
    print("Memory monitoring not supported, test skipped.")
else:
    print("Memory at startup", m0, 'kB')
    
    for type in ('CNA', 'COORD'):
        for haspot in (False, True):
            for replaceatoms in (False, True):
                print ("Running test: Type=%s HasPot=%s ReplaceAtoms=%s"
                       % (type, haspot, replaceatoms))
                leak = 0
                if not replaceatoms:
                    atoms = makeatoms()
                    if haspot:
                        atoms.set_calculator(asap3.EMT())
                        atoms.get_potential_energy()
                for i in range(steps):
                    if replaceatoms:
                        atoms = makeatoms()
                        if haspot:
                            atoms.set_calculator(asap3.EMT())
                            atoms.get_potential_energy()
                    if type == 'CNA':
                        asap3.analysis.CNA(atoms)
                    else:
                        asap3.analysis.CoordinationNumbers(atoms)
                   
                    m = int(asap3.heap_mallinfo())
                    if i == msteps:
                        m0 = m
                    if i % msteps == 0 and i > msteps:
                        print("    Memory usage:", m, "kB")
                        if m > m0:
                            leak = max(leak, m - m0)
                if leak > 1:
                    print("      *** MEMORY LEAK DETECTED ***")

                            
                            
                            



from __future__ import print_function
from numpy import *
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from ase import data
from asap3.testtools import *
import time

random.seed([114543, 8487])

listcutoff = 4.98409   # The EMT value for Cu
element = "Cu"
element2 = "Ag"
latconst = data.reference_states[data.atomic_numbers[element]]['a']
elementnumber = data.atomic_numbers[element]
element2number = data.atomic_numbers[element2]

def checkenergies(a, a2):
    err = 0
    if 0:
        print("    Checking Cartesian positions")
        err = abs((a.get_positions() - a2.get_positions()).ravel())
        idx = argmax(err)
        at, co = idx//3, idx%3
        print("      err =", err[idx], "at", idx, ":", at, co)
        ReportTest("      Worst Cartesian position (%d,%d)" % (at, co),
                   a.get_positions()[at,co],
                   a2.get_positions()[at,co], 1e-10)
    print("    Checking energies")
    e1 = a.get_potential_energies()
    e2 = a2.get_potential_energies()
    err = e1 - e2
    idx = argmax(err)
    ReportTest("      Worst energy (%s)" % (idx,), e1[idx], e2[idx], 1e-5)

for i in range(10):
    print()
print_version(1)
#print "RandomArray seeds:", RandomArray.get_seed()
if len(sys.argv) >= 2 and sys.argv[1] == '--pbc':
    if len(sys.argv) < 3:
        raise ValueError("No argument to --pbc")
    pbcch = sys.argv[2]
    if len(pbcch) != 3:
        raise ValueError("Argument to pbc should be a string of three digits.")
    pbc = []
    for i in (0,1,2):
        pbc.append(int(pbcch[i]))
    print("Periodic boundary conditions forced!")
else:
    pbc = tuple(random.randint(0,2,3))
print("Periodic boundaries:", pbc)

#Verbose(1)

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(10,10,10),
                          symbol=element, pbc=pbc)
atoms2 = atoms.copy()
atoms = MonteCarloAtoms(atoms)
print("Number of atoms:", len(atoms))
# Replace 10% of the atoms with element2
newz = where(greater(random.random((len(atoms),)), 0.9),
             element2number, elementnumber)
atoms.set_atomic_numbers(newz)
atoms2.set_atomic_numbers(newz)

atoms.set_calculator(MonteCarloEMT())
atoms2.set_calculator(EMT())
atoms.get_potential_energy()

for e in ((element,elementnumber), (element2,element2number)):
    print ("Number of %s atoms (Z=%d): %d"
           % (e[0], e[1], sum(equal(atoms.get_atomic_numbers(), e[1]))))

print() 
print("Testing perturbations of single atoms")

magnarr = array((0.0, 0.01, 0.1, 1.0, 3.0, 10.0))
numarr = array((1, 3, 10, len(atoms)//2, -3))
pick1 = argsort(random.random((len(magnarr),)))
for magn in take(magnarr, pick1):
    pick2 = argsort(random.random((len(numarr),)))
    for number in take(numarr, pick2):
        # Pick number random atoms.
        if number < 0:
            # Find N neighboring atoms and perturb them
            number = -number
            nblist = atoms.get_calculator().get_neighborlist()
            n = 0
            while len(nblist[n]) < number:
                n += 1
            pick = concatenate([array((n,)), nblist[n][:number-1]])
            del nblist
        else:
            pick = argsort(random.random((len(atoms),)))[:number]
        if number > 15:
            s = "<<< %s atoms >>>" % (number,)
        else:
            s = str(list(pick))
        print("  dr = %.3f: %d atom(s): %s" % (magn, number, s))
        for i in pick:
            dr = random.standard_normal(3)
            dr *= magn/sqrt(dot(dr,dr))
            atom = atoms[i]
            atom.position = atom.position + dr
            atom = atoms2[i]
            atom.position = atom.position + dr

        checkenergies(atoms, atoms2)

print()
print("Testing alchemy")

pick2 = argsort(random.random((len(numarr),)))
for number in take(numarr, pick2):
    # Pick number random atoms.
    if number < 0:
        # Find N neighboring atoms and perturb them
        number = -number
        nblist = atoms.get_calculator().get_neighborlist()
        n = 0
        while len(nblist[n]) < number:
            n += 1
        pick = concatenate([array((n,)), nblist[n][:number-1]])
        del nblist
    else:
        pick = argsort(random.random((len(atoms),)))[:number]
    if number > 15:
        s = "<<< %s atoms >>>" % (number,)
    else:
        s = str(list(pick))
        print("  Alchemy, %d atoms: %s" % (number, s))
        for i in pick:
            atom = atoms[i]
            atom2 = atoms2[i]
            z = atom.number
            assert z == atom2.number
            if (z == elementnumber):
                atom.number = element2number
                atom2.number = element2number
            else:
                assert z == element2number
                atom.number = elementnumber
                atom2.number = elementnumber
        checkenergies(atoms, atoms2)
        
ReportTest.Summary()




from __future__ import print_function
from ase.constraints import StrainFilter
from ase.optimize import MDMin
from asap3 import *
from ase.lattice.cubic import *
from asap3.testtools import ReportTest

print_version(1)
size = 5

atoms = FaceCenteredCubic(size=(size,size,size), symbol="Cu", pbc=True)
defsize = atoms.get_volume()
atoms.set_cell(atoms.get_cell() * 1.1, scale_atoms=True)
atoms.set_calculator(EMT())

def printvol(a):
    print("Volume:", a.get_volume(), " energy:",\
          atoms.get_potential_energy() + atoms.get_kinetic_energy(),\
          " stress:", atoms.get_stress()[:3])

f = StrainFilter(atoms, [1, 1, 1, 0, 0, 0])
opt = MDMin(f, logfile="/dev/null", dt=0.01/(atoms.get_cell()[0,0]))
printvol(atoms)
opt.attach(printvol, 10, atoms)
opt.run(0.01)
printvol(atoms)
print("Original vol:", defsize)
print()
stress = atoms.get_stress()
for i in range(6):
    ReportTest("Stress component %d (T=0)" % (i,), stress[0], 0.0, 1e-5)

atoms = FaceCenteredCubic(size=(size,size,size), symbol="Cu", pbc=True)
atoms.set_calculator(EMT())
dyn = Langevin(atoms, 10*units.fs, 500*units.kB, 0.02)
dyn.attach(printvol, 100, atoms)
dyn.run(500)

print()

f = StrainFilter(atoms, [1, 1, 1, 0, 0, 0])
opt = MDMin(f, logfile="/dev/null", dt=0.01/(atoms.get_cell()[0,0]))
printvol(atoms)
opt.attach(printvol, 10, atoms)
opt.run(0.01)
printvol(atoms)
print("Original vol:", defsize)
print()

stress = atoms.get_stress()
for i in range(6):
    ReportTest("Stress component %d (T=500K)" % (i,), stress[0], 0.0, 1e-5)

ReportTest.Summary()



from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from asap3 import NeighborCellLocator
from asap3.setup.multidislocation import random_dislocations
from asap3.setup.dislocation import Dislocation
from asap3.memory import get_max_memory
from ase.atoms import Atoms
from ase.lattice.cubic import FaceCenteredCubic, BodyCenteredCubic
import ase.data
import numpy as np
import scipy.spatial
from multiprocessing import Pool, cpu_count
from . import plane_utils
from . import quat_utils
import math
import sys
import os

###
###  Below are helper functions for make_nanocrystalline
###


def remove_close(atoms, min_dist):
    "Remove atoms that are too close"
    print("Removing atoms that are too close (d_cut = %.3f Angstrom)" % (min_dist,))
    remove = set()
    nblist = NeighborCellLocator(min_dist, atoms)
    # Loop over all pairs of too-close atoms
    for i in range(len(atoms)):
        for j in nblist[i]:
            # Pair (i,j) is too close
            if not (i in remove or j in remove):
                if len(remove) % 2:
                    remove.add(i)
                else:
                    remove.add(j)
    print("Removing %i atoms out of %i" % (len(remove), len(atoms)))
    del atoms[list(remove)]
    return atoms

def calculate_grain_bounding_planes(centres, size):

    a = (-1, 0, 1)
    ctr = [c for c in centres]
    for i in a:
        for j in a:
            for k in a:
                if i == j == k == 0: continue
                ctr += [size * [i, j, k] + c for c in centres]
    ctr = np.array(ctr)

    voronoi = scipy.spatial.Voronoi(ctr, furthest_site=False,
                                    incremental=False, qhull_options=None)
    regions = [voronoi.regions[voronoi.point_region[i]] for i in range(len(centres))]

    grain_cells = []
    for i, reg in enumerate(regions):

        cell_vertices = np.array([voronoi.vertices[v] for v in reg])
        hull = scipy.spatial.ConvexHull(cell_vertices)

        bounding_planes = [plane_utils.plane_equation(centres[i], h, cell_vertices)
                           for h in hull.simplices]
        grain_cells += [(cell_vertices, bounding_planes)]

    return grain_cells

_field_to_apply = None

def apply_fields(r):
    global _field_to_apply
    res = np.zeros_like(r)
    for f in _field_to_apply:
        res += f(r)
    return res

_positions_args = None
def create_layer_positions(i):
    global _positions_args
    y0, y1, z0, z1, unit = _positions_args
    positions = [np.dot((i, j, k), unit.cell) + atom_pos
                 for j in range(y0, y1+1)
                 for k in range(z0, z1+1)
                 for atom_pos in unit.get_positions()]
    return np.array(positions)

def create_atoms(centres, rotations, unit, size, min_dist, dislocations=None,
                 multiproc=False):
    "Create the nanocrystalline sample"
    global _field_to_apply, _positions_args

    if multiproc:
        if multiproc is True:
            multiproc_cpus = None  # Use default
        else:
            if not isinstance(multiproc, int) or multiproc <= 0:
                raise TypeError("multiproc argument must be a positive integer.")
            multiproc_cpus = multiproc  # Use specified number
            
    grain_cells = calculate_grain_bounding_planes(centres, size)

    ntotal = 0
    result = None
    for grain_num, (centre, rotation) in enumerate(zip(centres, rotations)):
        assert centre.shape == (3,)
        assert rotation.shape == (3,3)

        (bounding_vertices, bounding_planes) = grain_cells[grain_num]

        #find maximum extent in [x, y, z]
        U = rotation
        rot_basis = U / np.linalg.norm(U, axis=0)

        proj_vertices = np.dot(bounding_vertices, rot_basis)

        xmin = np.min(proj_vertices[:,0])
        ymin = np.min(proj_vertices[:,1])
        zmin = np.min(proj_vertices[:,2])

        xmax = np.max(proj_vertices[:,0])
        ymax = np.max(proj_vertices[:,1])
        zmax = np.max(proj_vertices[:,2])

        #calculate number of unit cells in each direction
        cubic_cell = [unit.cell[0][0], unit.cell[1][1], unit.cell[2][2]]

        x0 = int(math.floor(xmin / cubic_cell[0]))
        y0 = int(math.floor(ymin / cubic_cell[1]))
        z0 = int(math.floor(zmin / cubic_cell[2]))

        x1 = int(math.ceil(xmax / cubic_cell[0]))
        y1 = int(math.ceil(ymax / cubic_cell[1]))
        z1 = int(math.ceil(zmax / cubic_cell[2]))

        # If there are dislocations, make space for their worst-case
        # cumulative dislplacement
        if dislocations:
            field = dislocations[grain_num]
            n_disl = len(field)
            correction = int(n_disl // np.sqrt(2))
            x0 -= correction
            y0 -= correction
            z0 -= correction
            x1 += correction
            y1 += correction
            z1 += correction
            
        #create cube of atoms    #todo: outer product
        if multiproc:
            _positions_args = (y0, y1, z0, z1, unit)
            pool = Pool(multiproc_cpus)
            positions = pool.map(create_layer_positions, range(x0, x1+1))
            positions = np.concatenate(positions)
            pool.close()
        else:
            positions = [np.dot((i, j, k), unit.cell) + atom_pos
                         for i in range(x0, x1+1)
                         for j in range(y0, y1+1)
                         for k in range(z0, z1+1)
                         for atom_pos in unit.get_positions()]
            positions = np.array(positions)
        print("Temporary length of position array:", len(positions))
        sys.stdout.flush()
        # Apply dislocation field, if any
        if dislocations:
            centre_of_mass = positions.sum(axis=0)/len(positions)
            if multiproc:
                if multiproc_cpus:
                    ncores = multiproc_cpus
                else:
                    ncores = cpu_count()
                print("Using {} cores.".format(ncores))
                _field_to_apply = field  # Cannot pass a lambda to Pool.map
                arg = np.array_split(positions - centre_of_mass, ncores)
                pool = Pool(multiproc_cpus)
                res = pool.map(apply_fields, arg)
                totalfield = np.concatenate(res)
                pool.close()
                assert totalfield.shape == positions.shape
            else:
                totalfield = np.zeros_like(positions)
                for i, f in enumerate(field):
                    totalfield += f(positions - centre_of_mass)
            positions = positions + totalfield
            print(n_disl, "dislocations introduced.")
            sys.stdout.flush()
        # Apply rotation
        positions = np.dot(positions, U.transpose())
        
        #remove excess atoms
        print("Removing excess atoms.")
        in_cell = np.ones(len(positions)).astype(np.int8)
        for (p, n) in bounding_planes:
            in_cell = np.logical_and(in_cell, np.dot(n, (positions - p).T) > 0)

        positions = positions[in_cell]

        print("Creating Atoms object, and merging...")
        atoms = Atoms(positions = positions, cell=size, numbers = 29 * np.ones(len(positions)))
        atoms.set_tags(grain_num * np.ones(len(atoms), int))
        if result is None:
            result = atoms
            atoms.set_cell(size, scale_atoms=False)
            atoms.set_pbc(True)
        else:
            result.extend(atoms)
        print("grain %d contains %d atoms (max mem. %.1f GB)."
              % (grain_num, len(positions), get_max_memory()/1024))
        sys.stdout.flush()

    print("Total atoms:", len(result))
    result = remove_close(result, min_dist)
    print("Remaining atoms:", len(result))
    return result




from __future__ import print_function
# Checks for Ticket #11

from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest

print("Test for Ticket #11: https://trac.fysik.dtu.dk/projects/Asap/ticket/11")

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(6,6,6),
                          symbol="Cu")
atoms.set_calculator(EMT())
r = atoms.get_positions()
print("Orig position", r[-1])

uc = atoms.get_cell()
print(uc)
r[-1] = 1.51*uc[2]
atoms.set_positions(r)
print(atoms.get_potential_energy())

p1 = atoms.get_positions()[-1]
print("p1:", p1)

atoms.set_cell(uc, scale_atoms=True)
print(atoms.get_potential_energy())
p2  = atoms.get_positions()[-1]
print("p2:", p2)

atoms.set_cell(uc, scale_atoms=False)
print(atoms.get_potential_energy())
p3 = atoms.get_positions()[-1]
print("p3:", p3)

ReportTest("p2 equals p1", p2[2], p1[2], 1e-6)
ReportTest("p3 equals p1", p3[2], p1[2], 1e-6)
ReportTest.Summary()



"""Test local crystal structure analysis (CNA and Coordination number)."""
from __future__ import print_function


from asap3.analysis import RestrictedCNA, CoordinationNumbers, FullCNA
from asap3.testtools import ReportTest
from ase.lattice import bulk
import numpy as np

atoms = bulk('Cu')
atoms = atoms.repeat((10, 10, 10))
del atoms[100]   #Create point defect

print("Testing CNA")
RestrictedCNA(atoms)
t = atoms.get_tags()
counts = [0,] * 3
for i in range(3):
    counts[i] = (t == i).sum()
    print("CNA mode {0}: {1} atoms.".format(i, counts[i]))
    
ReportTest("CNA: fcc atoms", counts[0], len(atoms)-12, 0)
ReportTest("CNA: hcp atoms", counts[1], 0, 0)
ReportTest("CNA: other atoms", counts[2], 12, 0)

print()
print("Testing coordination number")
t = CoordinationNumbers(atoms)
counts = np.zeros(20, int)
wanted = np.zeros(20, int)
wanted[11] = 12
wanted[12] = len(atoms) - 12
for i in range(len(counts)):
    counts[i] = (t == i).sum()
    print("Coordination number {0}: {1} atoms.".format(i, counts[i]))
    ReportTest("Coordination number {0}".format(i), counts[i], wanted[i], 0)
    
print()
print("Testing Full CNA")
cnacalc = FullCNA(atoms)
cna, totalcna = cnacalc.get_normal_and_total_cna()
counts = {(4,2,1): 0, (3,1,1):0}
ReportTest("Length of CNA vector", len(cna), len(atoms), 0)
for i in range(len(atoms)):
    ReportTest("Number of CNA vectors (atom {0})".format(i), 
               sum(cna[i].values()), t[i], 0, silent=True)
    for key, value in cna[i].items():
        counts[key] += value
    if (t[i] == 12):
        ReportTest("Number of 421 bonds (atom {0})".format(i),
                   cna[i][(4,2,1)], 12, 0, silent=True)
    
ReportTest("Global 421 count", totalcna[(4,2,1)]*2, counts[(4,2,1)], 0)
ReportTest("Global 311 count", totalcna[(3,1,1)]*2, counts[(3,1,1)], 0)   


ReportTest.Summary()
